"""通用配置模块 - 简化的 JSON 配置基类."""

from __future__ import annotations

import atexit
import contextlib
import json
import logging
import os
import platform
import tempfile
import threading
from decimal import Decimal
from functools import cached_property
from pathlib import Path
from typing import Callable, ClassVar

try:
    import fcntl
    from fcntl import LOCK_EX  # type: ignore[attr-defined]

    HAS_FCNTL = True
except ImportError:
    HAS_FCNTL = False

__all__ = ["ConfigMixin"]

logger = logging.getLogger(__name__)


class _DefaultConfigMixin:
    @property
    def _HOME_DIR(self) -> Path:
        """获取用户主目录."""
        return Path.home()

    @property
    def _EXTENSION(self) -> str:
        """获取可执行文件扩展名."""
        return ".exe" if self._IS_WINDOWS else ""

    @property
    def _IS_WINDOWS(self) -> bool:
        """判断是否为 Windows 系统."""
        return platform.system() == "Windows"


class ConfigMixin(_DefaultConfigMixin):
    """简化的 JSON 配置混入基类.

    核心功能:
    - 自动从 JSON 文件加载配置
    - 支持动态设置属性并保存
    - 简单的属性访问代理

    安全特性:
    - 线程安全的配置读写
    - 循环引用检测和清理
    - 保存失败时的备份恢复机制
    - 多进程安全的文件锁机制
    """

    ROOT_DIR: ClassVar[Path] = Path.home() / ".pytola"
    NAME: str = ""

    _file_attrs: dict[str, object]
    _attrs: dict[str, object]
    _lock: threading.RLock

    def __init__(self) -> None:
        """初始化配置混入类."""
        self._file_attrs: dict[str, object] = {}
        self._attrs: dict[str, object] = {}
        self._observers: list = []

        # 使用可重入锁允许同一线程多次获取
        self._lock = threading.RLock()

        self.load_from_json()

        if not self.ROOT_DIR.exists():
            logger.debug(f"创建设置目录:{self.ROOT_DIR}")
            self.ROOT_DIR.mkdir(parents=True, exist_ok=True)

        atexit.register(self.save_to_json)

    @property
    def attrs(self) -> dict[str, object]:
        """获取所有配置属性."""
        return {k: v for k, v in self._attrs.items() if not k.startswith("_") and not callable(v)}

    @property
    def attrs_with_private(self) -> dict[str, object]:
        """获取所有配置属性,包括私有属性."""
        return {k: v for k, v in self._attrs.items() if not callable(v)}

    def __getattribute__(self, name: str) -> object:
        """获取属性,优先从实例属性中查找."""
        # 私有属性和特殊属性直接访问
        if name.startswith("_") or (name.startswith("__") and name.endswith("__")):
            return object.__getattribute__(self, name)

        # 从 _attrs 中查找配置属性
        attrs = object.__getattribute__(self, "_attrs")
        if name in attrs:
            value = attrs[name]
            # 只在调试级别输出详细日志
            if logger.isEnabledFor(logging.DEBUG):
                logger.debug(f"获取配置属性:{name} = {value!r}")
            return value

        return object.__getattribute__(self, name)

    def __setattr__(self, name: str, value: object) -> None:
        """设置属性."""
        if name.startswith("_"):
            object.__setattr__(self, name, value)
        else:
            with self._lock:
                old_value = self._attrs.get(name)
                self._attrs[name] = value

                # 通知观察者
                if old_value != value:
                    self._notify_observers(name, old_value, value)

    def add_observer(self, callback: Callable[[str, object, object], None]) -> None:
        """添加配置变更观察者."""
        with self._lock:
            if callback not in self._observers:
                self._observers.append(callback)
                if logger.isEnabledFor(logging.DEBUG):
                    logger.debug(
                        f"添加配置观察者:{callback.__name__ if hasattr(callback, '__name__') else 'anonymous'}",
                    )

    def remove_observer(self, callback: Callable[[str, object, object], None]) -> None:
        """移除配置变更观察者.

        Args:
            callback: 要移除的回调函数
        """
        with self._lock:
            if callback in self._observers:
                self._observers.remove(callback)
                if logger.isEnabledFor(logging.DEBUG):
                    logger.debug(
                        f"移除配置观察者:{callback.__name__ if hasattr(callback, '__name__') else 'anonymous'}",
                    )

    def _notify_observers(self, key: str, old_value: object, new_value: object) -> None:
        """通知所有观察者配置变更."""
        if not self._observers:
            return

        if logger.isEnabledFor(logging.DEBUG):
            logger.debug(f"通知 {len(self._observers)} 个观察者配置变更:{key}")

        try:
            for observer in self._observers[:]:  # 使用副本避免迭代时修改列表
                observer(key, old_value, new_value)
        except (OSError, RuntimeError, ValueError) as e:
            logger.error(f"观察者 {observer} 执行失败:{e}", exc_info=True)

    @property
    def _config_file(self) -> Path:
        """配置文件路径."""
        name = self.NAME or self._default_name
        return self.ROOT_DIR / f"{name}.json"

    @property
    def _backup_file(self) -> Path:
        """备份文件路径."""
        return self._config_file.with_suffix(".json.bak")

    @property
    def _default_name(self) -> str:
        """默认配置名称 (从类名转换)."""
        import re

        class_name = type(self).__name__
        name = re.sub(r"(.)([A-Z][a-z]+)", r"\1_\2", class_name)
        name = re.sub(r"([a-z0-9])([A-Z])", r"\1_\2", name).lower()
        return name.replace("_config", "")

    @cached_property
    def cls_attrs(self) -> dict[str, object]:
        """类属性字典."""
        return {
            attr: value
            for attr, value in self.__class__.__dict__.items()
            if not attr.startswith("_") and not callable(value)
        }

    def load_from_json(self) -> None:
        """从 JSON 文件加载配置.

        加载优先级:
        1. 文件中的配置值(最高优先级)
        2. 类属性默认值(最低优先级)
        """
        # 清空现有属性
        self._attrs.clear()
        self._file_attrs.clear()

        # 先加载类属性作为默认值
        self._attrs.update(self.cls_attrs)
        if logger.isEnabledFor(logging.DEBUG):
            logger.debug(f"加载类属性默认值:{self.cls_attrs}")

        # 如果配置文件不存在,直接返回
        if not self._config_file.exists():
            logger.debug(f"配置文件不存在,使用默认值:{self._config_file}")
            return

        try:
            with self._config_file.open("r", encoding="utf-8") as f:
                loaded_data = json.load(f)

            # 处理 null 值
            if loaded_data is None:
                logger.warning(f"配置文件内容为 null,忽略文件配置: {self._config_file}")
                return

            # 验证数据格式
            if not isinstance(loaded_data, dict):
                logger.error(
                    f"配置文件格式错误,期望 dict 类型但得到 {type(loaded_data).__name__}: {self._config_file}",
                )
                return

            self._file_attrs = loaded_data
            # 文件配置覆盖类属性(重要:这是用户期望的行为)
            self._attrs.update(self._file_attrs)
            if logger.isEnabledFor(logging.DEBUG):
                logger.debug(f"从配置文件加载并覆盖属性:{self._file_attrs}")

        except json.JSONDecodeError:
            logger.exception(f"配置文件 JSON 解析失败:{self._config_file}")
            self._restore_from_backup()
        except OSError as e:
            logger.exception(f"读取配置文件失败:{self._config_file}", exc_info=e)

        if logger.isEnabledFor(logging.DEBUG):
            logger.debug(f"最终加载的属性:{self._attrs}")

    def _restore_from_backup(self) -> None:
        """从备份文件恢复配置."""
        if self._backup_file.exists():
            logger.info(f"尝试从备份文件恢复:{self._backup_file}")
            try:
                with self._backup_file.open("r", encoding="utf-8") as f:
                    self._file_attrs = json.load(f) or {}
                logger.info("成功从备份文件恢复配置")
            except (json.JSONDecodeError, OSError) as e:
                logger.exception("从备份文件恢复失败", exc_info=e)

    def _atomic_write_config(self, data: dict) -> None:
        """原子写入配置文件,支持多进程并发安全.

        使用临时文件 + 重命名的方式确保写入的原子性:
        1. 在同一目录创建临时文件
        2. 写入数据到临时文件
        3. 使用原子重命名替换原文件

        在 Unix 系统上使用文件锁防止并发冲突.
        """
        config_dir = self._config_file.parent

        # 创建临时文件(在同一文件系统上以确保原子操作)
        fd, temp_path = tempfile.mkstemp(suffix=".tmp", dir=config_dir)
        temp_file = Path(temp_path)

        try:
            # 写入临时文件
            with os.fdopen(fd, "w", encoding="utf-8") as f:
                json.dump(data, f, indent=4, ensure_ascii=False, default=self._default_encoder)
                f.flush()
                os.fsync(f.fileno())  # 确保数据刷入磁盘

            # 在 Unix 系统上对临时文件加锁
            if HAS_FCNTL:
                with contextlib.suppress(OSError):
                    fcntl.flock(fd, LOCK_EX)  # type: ignore[attr-defined]

            # 原子替换原配置文件
            # Windows 上 replace 可能失败如果文件被占用,需要重试
            max_retries = 3
            for attempt in range(max_retries):
                try:
                    temp_file.replace(self._config_file)
                    break
                except PermissionError as e:
                    if attempt < max_retries - 1:
                        logger.debug(f"替换配置文件失败,{0.1 * (attempt + 1)}秒后重试:{e}")
                        import time

                        time.sleep(0.1 * (attempt + 1))
                    else:
                        raise

        except Exception:
            # 清理临时文件
            with contextlib.suppress(OSError):
                temp_file.unlink()
            raise

    def _default_encoder(self, obj: object) -> object:
        """自定义 JSON 编码器,处理不可序列化的类型."""
        # 处理复数类型
        if isinstance(obj, complex):
            return {"__complex__": True, "real": obj.real, "imag": obj.imag}
        # 处理 Decimal 类型
        if isinstance(obj, Decimal):
            return float(obj)
        # 处理集合类型
        if isinstance(obj, (set, frozenset)):
            return list(obj)
        # 处理字节类型
        if isinstance(obj, bytes):
            return obj.decode("utf-8", errors="replace")
        # 处理其他不可序列化类型,返回字符串表示
        return str(obj)

    def _remove_circular_refs(
        self,
        obj: object,
        seen: set[int] | None = None,
        depth: int = 0,
        max_depth: int = 50,
    ) -> object:
        """递归移除循环引用,限制最大深度避免栈溢出."""
        if seen is None:
            seen = set()

        # 限制递归深度,防止栈溢出
        if depth > max_depth:
            return "<max depth exceeded>"

        # 如果是可变对象,检查循环引用,避免无限递归
        if isinstance(obj, dict):
            obj_id = id(obj)
            if obj_id in seen:
                return "<circular reference>"
            seen.add(obj_id)
            result = {k: self._remove_circular_refs(v, seen, depth + 1, max_depth) for k, v in obj.items()}
            seen.discard(obj_id)
            return result

        if isinstance(obj, (list, tuple)):
            obj_id = id(obj)
            if obj_id in seen:
                return "<circular reference>"
            seen.add(obj_id)
            result = [self._remove_circular_refs(item, seen, depth + 1, max_depth) for item in obj]
            seen.discard(obj_id)
            return result
        return obj

    def _is_basic_serializable(self, obj: object) -> bool:
        """检查对象是否是基本可序列化类型.

        Args:
            obj: 要检查的对象

        Returns
        -------
            如果是基本可序列化类型返回 True,否则返回 False
        """
        return isinstance(obj, (str, int, float, bool, type(None), Decimal, complex, bytes))

    def _validate_container(
        self,
        obj: dict | list | tuple,
        path: str,
        seen: set[int],
        max_depth: int,
        current_depth: int,
    ) -> bool:
        """验证容器类型(dict/list/tuple)的可序列化性.

        Args:
            obj: 要验证的容器对象
            path: 当前对象的路径
            seen: 已访问对象的 ID 集合
            max_depth: 最大嵌套深度
            current_depth: 当前嵌套深度

        Returns
        -------
            如果可序列化返回 True,否则返回 False
        """
        obj_id = id(obj)
        if obj_id in seen:
            # 检测到循环引用,直接返回 False,不再继续遍历
            return False
        seen.add(obj_id)

        if isinstance(obj, dict):
            for k, v in obj.items():
                if not isinstance(k, str):
                    with contextlib.suppress(ValueError, RuntimeError, OSError):
                        logger.warning(f"字典键不是字符串:{k!r} at '{path}'")
                    return False
                if not self._validate_serializable(v, f"{path}.{k}", seen.copy(), max_depth, current_depth + 1):
                    return False
        else:  # list or tuple
            for i, item in enumerate(obj):
                if not self._validate_serializable(item, f"{path}[{i}]", seen.copy(), max_depth, current_depth + 1):
                    return False

        return True

    def _validate_serializable(
        self,
        obj: object,
        path: str = "",
        seen: set[int] | None = None,
        max_depth: int = 50,
        current_depth: int = 0,
    ) -> bool:
        """验证对象是否可以 JSON 序列化.

        Args:
            obj: 要验证的对象
            path: 当前对象的路径(用于错误报告)
            seen: 已访问对象的 ID 集合,用于检测循环引用
            max_depth: 最大嵌套深度
            current_depth: 当前嵌套深度
        """
        if seen is None:
            seen = set()

        # 限制递归深度,防止栈溢出
        if current_depth > max_depth:
            with contextlib.suppress(ValueError, RuntimeError, OSError):
                logger.error(f"对象嵌套过深 ({current_depth}/{max_depth}) at '{path}'")
            return False

        # 检查基本类型
        if self._is_basic_serializable(obj):
            return True

        # 检查容器类型
        if isinstance(obj, (dict, list, tuple)):
            result = self._validate_container(obj, path, seen, max_depth, current_depth)
            if not result:
                # 只在验证失败时记录一次错误,避免重复日志
                with contextlib.suppress(ValueError, RuntimeError, OSError):
                    logger.error(f"配置包含不可序列化的数据或循环引用 at '{path}'")
            return result

        # 其他类型会在编码时转换为字符串
        return True

    def save_to_json(self) -> None:
        """保存配置到 JSON 文件,支持多进程并发安全."""
        with self._lock:
            try:
                self._config_file.parent.mkdir(parents=True, exist_ok=True)

                # 过滤掉可调用对象
                serializable_attrs = {key: value for key, value in self._attrs.items() if not callable(value)}

                # 先处理循环引用(在验证之前处理,避免重复报错)
                cleaned_attrs = self._remove_circular_refs(serializable_attrs)

                # 验证数据可序列化性(可选,仅用于警告用户)
                if not self._validate_serializable(cleaned_attrs):
                    with contextlib.suppress(ValueError, RuntimeError, OSError):
                        logger.warning("配置包含不可完全序列化的数据,但仍会尝试保存")

                # 使用原子写入方式:先写入临时文件,再替换原文件
                # 这样可以避免多进程同时写入时的文件锁定问题
                self._atomic_write_config(cleaned_attrs)  # type: ignore[arg-type]

                logger.debug(f"配置已保存到:{self._config_file}")

            except (OSError, json.JSONDecodeError) as e:
                logger.exception(f"保存配置文件失败:{self._config_file}", exc_info=e)
                # 尝试从备份恢复(如果有备份的话)
                if self._backup_file.exists():
                    self._restore_from_backup()



"""Tests for the StateMachine implementation."""

from enum import Enum

import pytest

from pytola_core.patterns.statemachine import InvalidTransitionError, StateConfig, StateMachine, StateNotFoundError


class TestState(str, Enum):
    """测试用状态枚举."""

    __test__ = False  # 告诉 pytest 不要收集这个类作为测试

    IDLE = "idle"
    RUNNING = "running"
    PAUSED = "paused"
    STOPPED = "stopped"


class TestStateMachine:
    """测试 StateMachine 类."""

    def test_initial_state(self) -> None:
        """测试初始状态设置."""
        sm = StateMachine(initial_state=TestState.IDLE)
        assert sm.current_state == "idle"

    def test_force_set_state(self) -> None:
        """测试强制设置状态."""
        sm = StateMachine(initial_state=TestState.IDLE)
        sm.force_set_state(TestState.RUNNING)
        assert sm.current_state == "running"

    def test_register_state(self) -> None:
        """测试注册状态配置."""
        sm = StateMachine(initial_state=TestState.IDLE)
        sm.register_state(TestState.IDLE, StateConfig(enables={"start_btn"}, transitions={"start": TestState.RUNNING}))
        assert sm.has_state(TestState.IDLE)
        assert sm.state_config is not None
        assert "start_btn" in sm.state_config.enables

    def test_register_states(self) -> None:
        """测试批量注册状态."""
        sm = StateMachine(initial_state=TestState.IDLE)
        states_config: dict[TestState, StateConfig] = {
            TestState.IDLE: StateConfig(enables={"start_btn"}, transitions={"start": TestState.RUNNING}),
            TestState.RUNNING: StateConfig(
                enables={"pause_btn", "stop_btn"},
                transitions={"pause": TestState.PAUSED, "stop": TestState.STOPPED},
            ),
        }
        sm.register_states(states_config)  # type: ignore[arg-type]
        assert sm.has_state(TestState.IDLE)
        assert sm.has_state(TestState.RUNNING)

    def test_valid_transition(self) -> None:
        """测试有效的状态转换."""
        sm = StateMachine(initial_state=TestState.IDLE)
        sm.register_state(TestState.IDLE, StateConfig(enables={"start_btn"}, transitions={"start": TestState.RUNNING}))
        sm.register_state(TestState.RUNNING, StateConfig(enables={"stop_btn"}, transitions={}))

        assert sm.transition("start") is True
        assert sm.current_state == "running"

    def test_invalid_transition(self) -> None:
        """测试无效的状态转换."""
        sm = StateMachine(initial_state=TestState.IDLE)
        sm.register_state(TestState.IDLE, StateConfig(enables={"start_btn"}, transitions={"start": TestState.RUNNING}))

        # 尝试执行未定义的转换
        with pytest.raises(InvalidTransitionError) as exc_info:
            sm.transition("pause")

        assert exc_info.value.current_state == "idle"
        assert exc_info.value.attempted_action == "pause"

    def test_transition_to_unregistered_state(self) -> None:
        """测试转换到未注册的状态."""
        sm = StateMachine(initial_state=TestState.IDLE)
        sm.register_state(TestState.IDLE, StateConfig(enables={"start_btn"}, transitions={"start": TestState.RUNNING}))
        # 未注册 RUNNING 状态

        with pytest.raises(StateNotFoundError) as exc_info:
            sm.transition("start")

        assert exc_info.value.current_state == "idle"
        assert exc_info.value.attempted_action == "start"

    def test_state_change_callback(self) -> None:
        """测试状态变更回调."""
        callback_calls = []

        def on_state_change(old_state: str, new_state: str) -> None:
            callback_calls.append((old_state, new_state))

        sm = StateMachine(initial_state=TestState.IDLE, on_state_change=on_state_change)
        sm.register_state(TestState.IDLE, StateConfig(enables={"start_btn"}, transitions={"start": TestState.RUNNING}))
        sm.register_state(TestState.RUNNING, StateConfig(enables={"stop_btn"}, transitions={}))

        sm.transition("start")

        assert len(callback_calls) == 1
        assert callback_calls[0] == ("idle", "running")

    def test_can_transition(self) -> None:
        """测试检查是否可以转换."""
        sm = StateMachine(initial_state=TestState.IDLE)
        sm.register_state(TestState.IDLE, StateConfig(enables={"start_btn"}, transitions={"start": TestState.RUNNING}))

        assert sm.can_transition("start") is True
        assert sm.can_transition("pause") is False

    def test_get_available_actions(self) -> None:
        """测试获取可用动作列表."""
        sm = StateMachine(initial_state=TestState.IDLE)
        sm.register_state(
            TestState.IDLE,
            StateConfig(enables={"start_btn"}, transitions={"start": TestState.RUNNING, "reset": TestState.STOPPED}),
        )

        actions = sm.get_available_actions()
        assert set(actions) == {"start", "reset"}

    def test_transition_history(self) -> None:
        """测试状态转换历史记录."""
        sm = StateMachine(initial_state=TestState.IDLE)
        sm.register_state(TestState.IDLE, StateConfig(enables={"start_btn"}, transitions={"start": TestState.RUNNING}))
        sm.register_state(TestState.RUNNING, StateConfig(enables={"stop_btn"}, transitions={"stop": TestState.STOPPED}))
        sm.register_state(TestState.STOPPED, StateConfig(enables={"start_btn"}, transitions={}))

        sm.transition("start")
        sm.transition("stop")

        history = sm.get_transition_history()
        assert len(history) == 2
        assert history[0] == ("idle", "start", "running")
        assert history[1] == ("running", "stop", "stopped")

    def test_transition_history_limit(self) -> None:
        """测试限制历史记录数量."""
        sm = StateMachine(initial_state="state_0")
        # 注册多个状态和转换
        for i in range(5):
            state_name = f"state_{i}"
            next_state = f"state_{i + 1}" if i < 4 else "state_0"
            sm.register_state(state_name, StateConfig(enables=set(), transitions={"next": next_state}))

        # 执行多次转换
        for _ in range(4):
            sm.transition("next")

        # 只获取最近 2 条记录
        recent_history = sm.get_transition_history(limit=2)
        assert len(recent_history) == 2
        # 验证是最近的 2 条记录
        assert recent_history[0][0] == "state_2"  # 从 state_2 开始
        assert recent_history[1][0] == "state_3"  # 到 state_3

    def test_clear_history(self) -> None:
        """测试清空历史记录."""
        sm = StateMachine(initial_state=TestState.IDLE)
        sm.register_state(TestState.IDLE, StateConfig(enables={"start_btn"}, transitions={"start": TestState.RUNNING}))
        sm.register_state(TestState.RUNNING, StateConfig(enables={"stop_btn"}, transitions={}))

        sm.transition("start")
        sm.clear_history()

        assert len(sm.get_transition_history()) == 0

    def test_normalize_state_with_enum(self) -> None:
        """测试 Enum 状态的标准化."""
        sm = StateMachine(initial_state=TestState.IDLE)
        assert sm._normalize_state(TestState.RUNNING) == "running"
        assert sm._normalize_state("idle") == "idle"

    def test_repr(self) -> None:
        """测试字符串表示."""
        sm = StateMachine(initial_state=TestState.IDLE)
        sm.register_state(TestState.IDLE, StateConfig(enables={"start_btn"}, transitions={"start": TestState.RUNNING}))

        repr_str = repr(sm)
        assert "StateMachine" in repr_str
        assert "current_state='idle'" in repr_str
        assert "'idle'" in repr_str

    def test_callback_exception_handling(self) -> None:
        """测试回调函数异常处理."""

        def faulty_callback(old_state: str, new_state: str) -> None:
            msg = "Callback error"
            raise ValueError(msg)

        sm = StateMachine(initial_state=TestState.IDLE, on_state_change=faulty_callback)
        sm.register_state(TestState.IDLE, StateConfig(enables={"start_btn"}, transitions={"start": TestState.RUNNING}))
        sm.register_state(TestState.RUNNING, StateConfig(enables={"stop_btn"}, transitions={}))

        # 回调异常不应影响状态转换
        assert sm.transition("start") is True
        assert sm.current_state == "running"



"""Pytola Core - Essential utilities and components for Pytola applications.

This package provides core functionality including:
- Advanced logging system with colored output
- High DPI scaling support for Qt applications
- Security utilities for input validation
- Theme management system
- Environment configuration helpers
- Generic registry system for component management
- Finite state machine for state management.
- High-performance thread-safe SQLite database management.
"""

from pytola_core.logger import get_logger, logger
from pytola_core.patterns.statemachine import (
    InvalidTransitionError,
    StateConfig,
    StateMachine,
    StateMachineError,
    StateNotFoundError,
)
from pytola_core.persistance import DatabaseConnectionPool, DatabaseError, DatabaseManager, PoolConfig, TransactionError

__version__ = "0.1.25"


__all__ = [
    # Database
    "DatabaseConnectionPool",
    "DatabaseError",
    "DatabaseManager",
    # State machine
    "InvalidTransitionError",
    "PoolConfig",
    "StateConfig",
    "StateMachine",
    "StateMachineError",
    "StateNotFoundError",
    "TransactionError",
    # Logger
    "get_logger",
    "logger",
]



"""高性能 SQLite 数据库管理器 - 线程安全的数据库操作封装.

本模块提供:
- DatabaseConnectionPool: 线程安全的连接池
- DatabaseManager: 高级数据库操作管理器
- 支持上下文管理器和装饰器进行事务控制
"""

from __future__ import annotations

import sqlite3
import threading
import types
from contextlib import contextmanager
from dataclasses import dataclass, field
from datetime import datetime, timezone
from pathlib import Path
from queue import Empty, Queue
from typing import Callable, Generator, TypeVar

from ..logger import logger

__all__ = [
    "DatabaseConnectionPool",
    "DatabaseError",
    "DatabaseManager",
    "IsolationLevel",
    "RowFactory",
    "TransactionError",
]


# 类型别名
RowFactory = Callable[[sqlite3.Cursor], Callable[[sqlite3.Row], dict]]
IsolationLevel = str
T = TypeVar("T")


class DatabaseError(Exception):
    """数据库操作异常基类."""

    pass


class TransactionError(DatabaseError):
    """事务处理异常."""

    pass


@dataclass
class PoolConfig:
    """连接池配置.

    Attributes
    ----------
        max_size: 最大连接数
        min_size: 最小连接数
        timeout: 获取连接超时时间 (秒)
        recycle_timeout: 连接回收超时时间 (秒)
        check_connection_on_borrow: 借用连接时是否检查有效性
    """

    max_size: int = 10
    min_size: int = 2
    timeout: float = 30.0
    recycle_timeout: float = 3600.0  # 1 小时
    check_connection_on_borrow: bool = True


@dataclass
class ConnectionWrapper:
    """连接包装器，用于跟踪连接状态.

    Attributes
    ----------
        conn: SQLite 连接对象
        created_at: 创建时间戳
        last_used: 最后使用时间戳
        borrow_count: 借用次数
    """

    conn: sqlite3.Connection
    created_at: datetime = field(default_factory=lambda: datetime.now(timezone.utc))
    last_used: datetime = field(default_factory=lambda: datetime.now(timezone.utc))
    borrow_count: int = 0


class DatabaseConnectionPool:
    """线程安全的 SQLite 连接池.

    特性:
    - 使用 RLock 保证线程安全
    - 支持最小/最大连接数限制
    - 自动连接回收机制
    - 连接借用计数和统计
    - 支持上下文管理器

    示例:
    --------
    >>> pool = DatabaseConnectionPool(":memory:")
    >>> with pool.get_connection() as conn:
    ...     cursor = conn.execute("SELECT 1")
    """

    def __init__(self, db_path: str | Path, config: PoolConfig | None = None) -> None:
        """初始化连接池.

        Args:
            db_path: 数据库文件路径
            config: 连接池配置,None 则使用默认配置
        """
        self.db_path = Path(db_path)
        self.config = config or PoolConfig()
        self._pool: Queue[ConnectionWrapper] = Queue()
        self._lock = threading.RLock()
        self._total_connections = 0
        self._shutdown = False

        # 预创建最小连接数
        self._initialize_min_connections()

    def _try_initialize_single_connection(self) -> bool:
        """尝试初始化单个连接.

        Returns
        -------
            初始化成功返回 True,失败返回 False
        """
        try:
            conn = self._create_connection()
            self._pool.put(ConnectionWrapper(conn=conn))
            self._total_connections += 1
        except sqlite3.Error:
            logger.exception("初始化连接失败")
            return False
        else:
            return True

    def _initialize_min_connections(self) -> None:
        """预创建最小连接数的连接."""
        with self._lock:
            while self._total_connections < self.config.min_size:
                if not self._try_initialize_single_connection():
                    break

    def _create_connection(self) -> sqlite3.Connection:
        """创建新的数据库连接.

        Returns
        -------
            新创建的 SQLite 连接
        """
        conn = sqlite3.connect(
            str(self.db_path),
            check_same_thread=False,  # 允许跨线程使用(由连接池管理)
            isolation_level=None,  # 自动提交模式,由事务管理器控制
        )
        # 启用 WAL 模式以获得更好的并发性能
        conn.execute("PRAGMA journal_mode=WAL")
        # 设置 busy timeout
        conn.execute("PRAGMA busy_timeout=5000")
        # 启用外键约束
        conn.execute("PRAGMA foreign_keys=ON")
        return conn

    def _is_connection_valid(self, wrapper: ConnectionWrapper) -> bool:
        """检查连接是否有效.

        Args:
            wrapper: 连接包装器

        Returns
        -------
            连接有效返回 True,否则返回 False
        """
        try:
            # 检查连接是否可以执行简单查询
            wrapper.conn.execute("SELECT 1")
            # 检查连接是否超时
            if self.config.recycle_timeout > 0:
                age = (datetime.now(timezone.utc) - wrapper.created_at).total_seconds()
                if age > self.config.recycle_timeout:
                    return False
        except sqlite3.Error:
            return False
        else:
            return True

    def _return_connection(self, wrapper: ConnectionWrapper) -> None:
        """归还连接到连接池.

        Args:
            wrapper: 连接包装器
        """
        with self._lock:
            if self._shutdown:
                self._close_connection(wrapper)
                return

            # 检查连接是否需要回收
            if not self._is_connection_valid(wrapper):
                self._close_connection(wrapper)
                # 创建新连接替换
                try:
                    new_conn = self._create_connection()
                    self._pool.put(ConnectionWrapper(conn=new_conn))
                except sqlite3.Error:
                    logger.exception("创建替换连接失败")
                    with self._lock:
                        self._total_connections -= 1
                return

            # 更新最后使用时间并归还到池中
            wrapper.last_used = datetime.now(timezone.utc)
            self._pool.put(wrapper)

    def _close_connection(self, wrapper: ConnectionWrapper) -> None:
        """关闭连接.

        Args:
            wrapper: 连接包装器
        """
        try:
            wrapper.conn.close()
        except sqlite3.Error:
            logger.exception("关闭连接失败")

    def _try_get_connection_from_pool(self) -> ConnectionWrapper | None:
        """尝试从连接池获取连接,如果失败则返回 None.

        Returns
        -------
            获取到的连接包装器,如果无法获取则返回 None
        """
        try:
            wrapper = self._pool.get_nowait()
            if self._is_connection_valid(wrapper):
                wrapper.borrow_count += 1
                return wrapper
            # 连接无效,关闭并创建新的
            self._close_connection(wrapper)
            with self._lock:
                self._total_connections -= 1
        except (sqlite3.Error, Empty):
            # 池为空或获取失败
            return None
        else:
            return None

    def _try_create_new_connection(self) -> ConnectionWrapper | None:
        """尝试创建新连接.

        Returns
        -------
            创建成功的连接包装器,如果失败则返回 None
        """
        try:
            conn = self._create_connection()
            return ConnectionWrapper(conn=conn, borrow_count=1)
        except sqlite3.Error:
            logger.exception("创建新连接失败")
            return None

    def _acquire_connection_with_wait(self) -> ConnectionWrapper:
        """在等待后获取连接.

        Returns
        -------
            获取到的连接包装器

        Raises
        ------
            DatabaseError: 当无法获取连接时
        """
        try:
            wrapper = self._pool.get(timeout=self.config.timeout)
        except Empty as e:
            msg = f"获取连接超时 ({self.config.timeout}秒)"
            raise DatabaseError(msg) from e

        if not self._is_connection_valid(wrapper):
            self._close_connection(wrapper)
            with self._lock:
                self._total_connections -= 1
            msg = "连接无效,重新获取"
            raise DatabaseError(msg)
        wrapper.borrow_count += 1
        return wrapper

    def _process_acquired_connection(self, wrapper: ConnectionWrapper | None) -> tuple[ConnectionWrapper | None, bool]:
        """处理已获取的连接.

        Args:
            wrapper: 连接包装器

        Returns
        -------
            tuple[Optional[ConnectionWrapper], bool]: (连接包装器,是否继续循环)
        """
        if wrapper is not None:
            return wrapper, False

        # 池为空,尝试创建新连接
        if self._total_connections < self.config.max_size:
            wrapper = self._try_create_new_connection()
            if wrapper is not None:
                with self._lock:
                    self._total_connections += 1
                return wrapper, False

        # 达到最大连接数,需要等待
        return None, True

    @contextmanager
    def get_connection(self) -> Generator[sqlite3.Connection, None, None]:
        """获取数据库连接的上下文管理器.

        Yields
        ------
            SQLite 连接对象

        Raises
        ------
            DatabaseError: 当无法获取连接时
        """
        if self._shutdown:
            msg = "连接池已关闭"
            raise DatabaseError(msg)

        wrapper: ConnectionWrapper | None = None

        try:
            # 尝试从池中获取连接
            with self._lock:
                while True:
                    # 尝试从池中获取有效连接
                    wrapper = self._try_get_connection_from_pool()
                    wrapper, should_continue = self._process_acquired_connection(wrapper)
                    if not should_continue:
                        break

                    # 需要等待其他线程归还连接
                    self._lock.release()
                    try:
                        wrapper = self._acquire_connection_with_wait()
                        break
                    except DatabaseError as e:
                        # 连接无效或超时,继续重试
                        if "超时" in str(e):
                            # 超时错误,重新抛出
                            raise
                        # 连接无效,继续重试
                        continue
                    finally:
                        self._lock.acquire()

            if wrapper is None:
                msg = "无法获取有效连接"
                raise DatabaseError(msg)

            yield wrapper.conn

        finally:
            # 归还连接
            if wrapper is not None:
                self._return_connection(wrapper)

    def get_stats(self) -> dict:
        """获取连接池统计信息.

        Returns
        -------
            包含连接池状态的字典
        """
        with self._lock:
            pool_size = self._pool.qsize()
            return {
                "total_connections": self._total_connections,
                "available_connections": pool_size,
                "in_use_connections": self._total_connections - pool_size,
                "max_size": self.config.max_size,
                "min_size": self.config.min_size,
                "db_path": str(self.db_path),
            }

    def close(self) -> None:
        """关闭连接池并释放所有连接."""
        with self._lock:
            self._shutdown = True
            while not self._pool.empty():
                wrapper = self._pool.get_nowait()
                self._close_connection(wrapper)
            self._total_connections = 0
            logger.info(f"连接池已关闭:{self.db_path}")

    def __enter__(self) -> DatabaseConnectionPool:
        """进入上下文管理器."""
        return self

    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: types.TracebackType | None,
    ) -> None:
        """退出上下文管理器."""
        self.close()


class DatabaseManager:
    """高级数据库管理器,提供便捷的 CRUD 操作.

    特性:
    - 基于连接池的线程安全操作
    - 装饰器和上下文管理器风格的事务控制
    - 字典形式的结果集返回
    - 批量操作支持
    - 完整的类型注解

    示例:
    --------
    >>> db = DatabaseManager(":memory:")
    >>> db.execute('''
    ...     CREATE TABLE users (
    ...         id INTEGER PRIMARY KEY,
    ...         name TEXT NOT NULL,
    ...         email TEXT UNIQUE
    ...     )
    ... ''')
    >>> db.insert('users', {'name': 'Alice', 'email': 'alice@example.com'})
    >>> users = db.fetch_all('SELECT * FROM users')
    >>> db.close()
    """

    def __init__(self, db_path: str | Path, pool_config: PoolConfig | None = None) -> None:
        """初始化数据库管理器.

        Args:
            db_path: 数据库文件路径
            pool_config: 连接池配置,None 则使用默认配置
        """
        self.pool = DatabaseConnectionPool(db_path, pool_config)
        # 默认使用字典形式的行工厂
        self._row_factory: Callable[[sqlite3.Cursor], Callable[[sqlite3.Row], dict]] | None = self.dict_row_factory

    @property
    def row_factory(self) -> RowFactory | None:
        """获取行工厂函数."""
        return self._row_factory

    @row_factory.setter
    def row_factory(self, factory: RowFactory | None) -> None:
        """设置行工厂函数.

        Args:
            factory: 行工厂函数,接收 cursor 返回 dict
        """
        self._row_factory = factory

    @staticmethod
    def dict_row_factory(cursor: sqlite3.Cursor) -> Callable[[sqlite3.Row], dict]:
        """字典形式的行工厂.

        Args:
            cursor: SQLite 游标

        Returns
        -------
            将行转换为字典的函数
        """

        def row_factory(row: sqlite3.Row) -> dict:
            return dict(zip([description[0] for description in cursor.description], row))

        return row_factory

    def execute(
        self,
        query: str,
        params: tuple | dict | None = None,
        *,
        fetch: bool = False,
        commit: bool = False,
    ) -> list[dict] | int:
        """执行 SQL 查询.

        Args:
            query: SQL 查询语句
            params: 查询参数
            fetch: 是否返回结果
            commit: 是否提交事务

        Returns
        -------
            如果 fetch=True,返回结果列表;否则返回受影响的行数
        """
        with self.pool.get_connection() as conn:
            if self._row_factory:
                conn.row_factory = sqlite3.Row

            cursor = conn.cursor()

            try:
                if params:
                    cursor.execute(query, params)
                else:
                    cursor.execute(query)

                if commit:
                    conn.commit()

                if fetch:
                    rows = cursor.fetchall()
                    if self._row_factory and cursor.description:
                        # 使用 row factory 转换为字典
                        row_factory_func = self._row_factory(cursor)
                        return [row_factory_func(row) for row in rows]
                    return rows
            except sqlite3.Error as e:
                conn.rollback()
                msg = f"执行查询失败:{e}"
                raise DatabaseError(msg) from e
            else:
                return cursor.rowcount

    def fetch_one(self, query: str, params: tuple | dict | None = None) -> dict | None:
        """获取单条记录.

        Args:
            query: SQL 查询语句
            params: 查询参数

        Returns
        -------
            单条记录的字典形式,如果没有记录则返回 None
        """
        results = self.execute(query, params, fetch=True)
        if isinstance(results, list):
            return results[0] if results else None
        msg = "fetch_one 期望列表类型的返回值"
        raise DatabaseError(msg)

    def fetch_all(self, query: str, params: tuple | dict | None = None) -> list[dict]:
        """获取所有记录.

        Args:
            query: SQL 查询语句
            params: 查询参数

        Returns
        -------
            所有记录的字典列表
        """
        results = self.execute(query, params, fetch=True)
        if isinstance(results, list):
            return results
        msg = "fetch_all 期望列表类型的返回值"
        raise DatabaseError(msg)

    def insert(self, table: str, data: dict) -> int:
        """插入单条记录.

        Args:
            table: 表名
            data: 列名到值的映射

        Returns
        -------
            插入行的 rowid
        """
        columns = ", ".join(data.keys())
        placeholders = ", ".join(["?" for _ in data])
        query = f"INSERT INTO {table} ({columns}) VALUES ({placeholders})"

        with self.pool.get_connection() as conn:
            cursor = conn.cursor()
            try:
                cursor.execute(query, list(data.values()))
                conn.commit()
            except sqlite3.Error as e:
                conn.rollback()
                msg = f"插入记录失败:{e}"
                raise DatabaseError(msg) from e
            else:
                lastrowid = cursor.lastrowid
                return lastrowid if lastrowid is not None else 0

    def insert_many(self, table: str, data_list: list[dict]) -> int:
        """批量插入记录.

        Args:
            table: 表名
            data_list: 数据字典列表

        Returns
        -------
            插入的行数
        """
        if not data_list:
            return 0

        # 确保所有字典有相同的键
        columns = list(data_list[0].keys())
        placeholders = ", ".join(["?" for _ in columns])
        query = f"INSERT INTO {table} ({', '.join(columns)}) VALUES ({placeholders})"

        with self.pool.get_connection() as conn:
            cursor = conn.cursor()
            try:
                values = [[data.get(col) for col in columns] for data in data_list]
                cursor.executemany(query, values)
                conn.commit()
            except sqlite3.Error as e:
                conn.rollback()
                msg = f"批量插入失败:{e}"
                raise DatabaseError(msg) from e
            else:
                return cursor.rowcount

    def count(self, table: str, where: str, where_params: tuple | None = None) -> int:
        """统计记录数.

        Args:
            table: 表名
            where: WHERE 子句(不包含 WHERE 关键字),使用占位符如 column=?
            where_params: WHERE 子句的参数值元组

        Returns
        -------
            记录数

        Raises
        ------
            DatabaseError: 当参数验证失败时
        """
        if where:
            query = f"SELECT COUNT(*) FROM {table} WHERE {where}"
            params = where_params or ()
            result = self.fetch_one(query, params)
        else:
            query = f"SELECT COUNT(*) FROM {table}"
            result = self.fetch_one(query)

        return result["COUNT(*)"] if result else 0

    def query(self, query: str, params: tuple | None = None) -> list[dict]:
        """执行 SELECT 查询并返回结果.

        为了向后兼容而提供的便捷方法,封装了 fetch_all.

        Args:
            query: SQL SELECT 查询
            params: 查询参数

        Returns
        -------
            结果字典列表
        """
        return self.fetch_all(query, params)

    def query_one(self, query: str, params: tuple | None = None) -> dict | None:
        """执行 SELECT 查询并返回单个结果.

        为了向后兼容而提供的便捷方法,封装了 fetch_one.

        Args:
            query: SQL SELECT 查询
            params: 查询参数

        Returns
        -------
            单个结果字典,如果没有结果则返回 None
        """
        return self.fetch_one(query, params)

    def update(self, table: str, data: dict, where: str, where_params: tuple | None = None) -> int:
        """更新记录.

        Args:
            table: 表名
            data: 要更新的列和值
            where: WHERE 子句(不包含 WHERE 关键字),使用占位符如 column=?
            where_params: WHERE 子句的参数值元组

        Returns
        -------
            受影响的行数

        Raises
        ------
            DatabaseError: 当参数验证失败时
        """
        # 验证表名(只允许字母,数字,下划线)
        if not all(c.isalnum() or c == "_" for c in table):
            msg = f"无效的表名:{table}"
            raise DatabaseError(msg)

        set_clause = ", ".join([f"{col} = ?" for col in data])
        query = f"UPDATE {table} SET {set_clause} WHERE {where}"

        params = list(data.values())
        if where_params:
            params.extend(where_params)

        with self.pool.get_connection() as conn:
            cursor = conn.cursor()
            try:
                cursor.execute(query, tuple(params))
                conn.commit()
            except sqlite3.Error as e:
                conn.rollback()
                msg = f"更新记录失败:{e}"
                raise DatabaseError(msg) from e
            else:
                return cursor.rowcount

    def delete(self, table: str, where: str, where_params: tuple | None = None) -> int:
        """删除记录.

        Args:
            table: 表名
            where: WHERE 子句(不包含 WHERE 关键字),使用占位符如 column=?
            where_params: WHERE 子句的参数值元组

        Returns
        -------
            删除的行数

        Raises
        ------
            DatabaseError: 当参数验证失败时
        """
        # 验证表名(只允许字母,数字,下划线)
        if not all(c.isalnum() or c == "_" for c in table):
            msg = f"无效的表名:{table}"
            raise DatabaseError(msg)

        query = f"DELETE FROM {table} WHERE {where}"

        with self.pool.get_connection() as conn:
            cursor = conn.cursor()
            try:
                if where_params:
                    cursor.execute(query, where_params)
                else:
                    cursor.execute(query)
                conn.commit()
            except sqlite3.Error as e:
                conn.rollback()
                msg = f"删除记录失败:{e}"
                raise DatabaseError(msg) from e
            else:
                return cursor.rowcount

    @contextmanager
    def transaction(self) -> Generator[sqlite3.Connection, None, None]:
        """事务上下文管理器.

        Yields
        ------
            数据库连接对象,在事务中

        Example:
        --------
        >>> with db.transaction() as conn:
        ...     conn.execute("INSERT INTO users (name) VALUES (?)", ("Alice",))
        ...     conn.execute("INSERT INTO users (name) VALUES (?)", ("Bob",))
        ...     # 自动提交,如果抛出异常则回滚
        """
        with self.pool.get_connection() as conn:
            try:
                # 在自动提交模式下,需要显式开始事务
                conn.execute("BEGIN")
                yield conn
                conn.commit()
            except sqlite3.Error as e:
                conn.rollback()
                msg = f"事务执行失败:{e}"
                raise TransactionError(msg) from e

    def transactional(self, func: Callable[..., T]) -> Callable[..., T]:
        """事务装饰器.

        Args:
            func: 要装饰的函数,可以接受 conn 作为第一个参数或不接受

        Returns
        -------
            包装后的函数,自动在事务中执行

        Example:
        --------
        >>> @db.transactional
        ... def my_func(conn):  # 带连接参数
        ...     conn.execute("INSERT INTO test VALUES (1, ?)", ("Alice",))

        >>> @db.transactional
        ... def my_func():  # 不带连接参数
        ...     db.insert("test", {"name": "Alice"})
        """

        def wrapper(*args: object, **kwargs: object) -> T:
            with self.transaction() as conn:
                # 检查函数是否接受 conn 参数
                import inspect

                sig = inspect.signature(func)
                params = list(sig.parameters.values())

                if params and params[0].name == "conn":
                    # 函数接受 conn 参数
                    return func(conn, *args[1:] if args else (), **kwargs)
                # 函数不接受 conn 参数
                return func(*args, **kwargs)

        return wrapper

    def table_exists(self, table_name: str) -> bool:
        """检查表是否存在.

        Args:
            table_name: 表名

        Returns
        -------
            表存在返回 True,否则返回 False
        """
        query = "SELECT name FROM sqlite_master WHERE type='table' AND name=?"
        result = self.fetch_one(query, (table_name,))
        return result is not None

    def get_table_info(self, table_name: str) -> list[dict]:
        """获取表结构信息.

        Args:
            table_name: 表名

        Returns
        -------
            表结构信息的字典列表

        Raises
        ------
            DatabaseError: 当表名无效时
        """
        # 验证表名(只允许字母,数字,下划线)
        if not all(c.isalnum() or c == "_" for c in table_name):
            msg = f"无效的表名:{table_name}"
            raise DatabaseError(msg)

        query = f"PRAGMA table_info({table_name})"
        return self.fetch_all(query)

    def get_stats(self) -> dict:
        """获取数据库统计信息.

        Returns
        -------
            包含数据库和连接池状态的字典
        """
        stats = self.pool.get_stats()

        # 添加数据库基本信息
        with self.pool.get_connection() as conn:
            cursor = conn.cursor()
            cursor.execute("SELECT COUNT(*) FROM sqlite_master WHERE type='table'")
            table_count = cursor.fetchone()[0]
            stats["table_count"] = table_count

        return stats

    def close(self) -> None:
        """关闭数据库管理器并释放连接池."""
        self.pool.close()

    def __enter__(self) -> DatabaseManager:
        """进入上下文管理器."""
        return self

    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: types.TracebackType | None,
    ) -> None:
        """退出上下文管理器."""
        self.close()



"""Pytola 的日志模块.

本模块提供一个简单的、类似 loguru 的日志系统，具有彩色输出、
文件轮转和易于使用的 API。
"""

from __future__ import annotations

import contextlib
import logging
import logging.handlers
import sys
from pathlib import Path
from typing import ClassVar, Final

__all__ = ["get_logger", "logger"]

# Log file configuration
LOG_DIR: Final[Path] = Path.home() / ".pytola" / "logs"
LOG_FILE: Final[Path] = LOG_DIR / "pytola.log"
MAX_BYTES: Final[int] = 10 * 1024 * 1024  # 10MB
BACKUP_COUNT: Final[int] = 5


class _Logger:
    """一个简单的、类似 loguru 的日志器.

    提供干净、最小化的 API，用于带有自动控制台颜色和文件轮转的日志记录。

    示例
    --------
        >>> from pytola_core.logger import logger
        >>> logger.info("Hello, World!")
        >>> logger.debug("Debug message")  # 仅在启用调试时显示
        >>> logger.error("Error occurred")
    """

    _instance: logging.Logger | None = None

    COLORS: ClassVar[dict[str, str]] = {
        "DEBUG": "\033[36m",  # CYAN
        "INFO": "\033[32m",  # GREEN
        "WARNING": "\033[33m",  # YELLOW
        "ERROR": "\033[31m",  # RED
        "CRITICAL": "\033[41m",  # White on Red Background
        "RESET": "\033[0m",  # RESET COLOR
    }

    @classmethod
    def setup(cls, name: str = "pytola", level: int = logging.INFO) -> logging.Logger:
        """设置带有控制台和文件处理器的日志器.

        此方法应在应用程序启动时调用一次。
        后续调用将返回现有的日志器实例。

        参数
        ----------
        name : str
            日志器名称。默认为 "pytola"
        level : int
            最低日志级别。默认为 INFO
            使用 logger.DEBUG 获取更详细的输出

        返回
        -------
        logging.Logger
            配置好的带有控制台和文件处理器的日志器

        示例
        --------
            >>> from pytola_core.logger import Logger
            >>> logger = Logger.setup(level=Logger.DEBUG)
        """
        if cls._instance is not None:
            return cls._instance

        # Ensure log directory exists
        with contextlib.suppress(OSError):
            LOG_DIR.mkdir(parents=True, exist_ok=True)

        class _ColoredFormatter(logging.Formatter):
            """自定义格式化器，为日志消息添加 ANSI 颜色."""

            def format(self, record: logging.LogRecord) -> str:
                """使用适当的颜色格式化日志记录."""
                # 根据日志级别应用颜色
                log_color = cls.COLORS.get(record.levelname, cls.COLORS["RESET"])

                # 首先格式化消息
                message = super().format(record)

                # 将整个消息包装在级别颜色中
                return f"{log_color}{message}{cls.COLORS['RESET']}"

        # 创建日志器
        logger = logging.getLogger(name)
        logger.setLevel(logging.DEBUG)  # 捕获所有级别，在处理器级别过滤

        console_handler = logging.StreamHandler(sys.stdout)
        console_handler.setLevel(level)
        console_formatter = _ColoredFormatter("%(asctime)s | %(levelname)-8s | %(message)s", datefmt="%H:%M:%S")
        console_handler.setFormatter(console_formatter)

        # 文件处理器 - 捕获所有 DEBUG+ 日志并带轮转功能
        file_handler = logging.handlers.RotatingFileHandler(
            LOG_FILE,
            maxBytes=MAX_BYTES,
            backupCount=BACKUP_COUNT,
            encoding="utf-8",
        )
        file_handler.setLevel(logging.DEBUG)
        file_formatter = logging.Formatter(
            "%(asctime)s - %(name)s - %(levelname)s - %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S",
        )
        file_handler.setFormatter(file_formatter)

        # 添加处理器
        logger.addHandler(console_handler)
        logger.addHandler(file_handler)

        # 防止多次添加处理器
        logger.propagate = False

        cls._instance = logger
        return logger

    @classmethod
    def get_logger(cls, name: str = "pytola") -> logging.Logger:
        """获取日志器实例.

        如果日志器尚未设置，此方法将使用默认设置进行设置。
        如需自定义配置，请改用 `Logger.setup()`。

        参数:
            name: 日志器名称。默认为 "pytola"

        返回
        -------
            配置好的日志器实例
        """
        if cls._instance is None:
            return cls.setup(name)
        return cls._instance


# 预配置的日志器实例，便于导入
logger: logging.Logger = _Logger.get_logger()

# 为日志器对象添加便利属性（类似 loguru）
# 这些是动态添加的，以便更容易访问日志级别
setattr(logger, "DEBUG", logging.DEBUG)  # noqa: B010
setattr(logger, "INFO", logging.INFO)  # noqa: B010
setattr(logger, "WARNING", logging.WARNING)  # noqa: B010
setattr(logger, "ERROR", logging.ERROR)  # noqa: B010
setattr(logger, "CRITICAL", logging.CRITICAL)  # noqa: B010

get_logger = _Logger.get_logger



"""通用有限状态机实现 - 基于配置的声明式状态机."""

from __future__ import annotations

from dataclasses import dataclass, field
from enum import Enum
from typing import Any, Callable

from ..logger import logger


@dataclass
class StateConfig:
    """状态配置数据类.

    Attributes
    ----------
        enables: 启用的组件名称集合
        transitions: 允许的状态转换 {action: target_state}
        metadata: 额外的元数据(可选)
    """

    enables: set[str] = field(default_factory=set)
    transitions: dict[str, str] = field(default_factory=dict)
    metadata: dict[str, Any] = field(default_factory=dict)


class StateMachineError(Exception):
    """状态机异常基类.

    Args:
        message: 错误消息
        current_state: 当前状态(可选)
        attempted_action: 尝试的动作(可选)
    """

    def __init__(self, message: str, current_state: str | None = None, attempted_action: str | None = None) -> None:
        super().__init__(message)
        self.current_state = current_state
        self.attempted_action = attempted_action

    def __str__(self) -> str:
        """返回错误消息字符串."""
        parts = [super().__str__()]
        if self.current_state:
            parts.append(f"当前状态:{self.current_state}")
        if self.attempted_action:
            parts.append(f"尝试动作:{self.attempted_action}")
        return ", ".join(parts)


class InvalidTransitionError(StateMachineError):
    """无效的状态转换异常."""

    pass


class StateNotFoundError(StateMachineError):
    """状态未找到异常."""

    pass


class StateMachine:
    """声明式有限状态机.

    特性:
    - 基于配置的声明式状态定义
    - 支持状态转换验证
    - 支持状态变更回调
    - 线程安全的状态访问
    - 详细的状态转换日志

    Example:
        >>> # 定义状态枚举
        >>> class ScannerState(str, Enum):
        ...     INIT = "init"
        ...     COUNTING = "counting"
        ...     SEARCHING = "searching"
        ...     FINISHED = "finished"
        >>>
        >>> # 创建状态机
        >>> sm = StateMachine(initial_state=ScannerState.INIT)
        >>>
        >>> # 注册状态配置
        >>> sm.register_state(
        ...     ScannerState.INIT,
        ...     StateConfig(
        ...         enables={"open_btn", "config_btn"},
        ...         transitions={"start_counting": ScannerState.COUNTING}
        ...     )
        ... )
        >>>
        >>> # 执行状态转换
        >>> sm.transition("start_counting")
        >>> print(sm.current_state)  # counting
    """

    def __init__(self, initial_state: str | Enum, on_state_change: Callable[[str, str], None] | None = None) -> None:
        """初始化状态机.

        Args:
            initial_state: 初始状态
            on_state_change: 状态变更回调函数 (old_state, new_state)
        """
        self._current_state = self._normalize_state(initial_state)
        self._states: dict[str, StateConfig] = {}
        self._on_state_change = on_state_change
        self._transition_history: list[tuple[str, str, str]] = []  # (from, action, to)

    @staticmethod
    def _normalize_state(state: str | Enum) -> str:
        """将状态标准化为字符串.

        Args:
            state: 状态值(字符串或 Enum)

        Returns
        -------
            标准化的状态字符串
        """
        if isinstance(state, Enum):
            return state.value if hasattr(state, "value") else state.name
        return state

    def register_state(self, state: str | Enum, config: StateConfig) -> None:
        """注册状态配置.

        Args:
            state: 状态标识
            config: 状态配置
        """
        state_str = self._normalize_state(state)
        self._states[state_str] = config
        logger.debug(f"注册状态:{state_str}, 配置:{config}")

    def register_states(self, states_config: dict[str | Enum, StateConfig]) -> None:
        """批量注册状态配置.

        Args:
            states_config: 状态配置字典 {state: StateConfig}
        """
        for state, config in states_config.items():
            self.register_state(state, config)

    @property
    def current_state(self) -> str:
        """获取当前状态.

        Returns
        -------
            当前状态字符串
        """
        return self._current_state

    @property
    def state_config(self) -> StateConfig | None:
        """获取当前状态的配置.

        Returns
        -------
            当前状态的配置,如果状态未注册则返回 None
        """
        return self._states.get(self._current_state)

    def has_state(self, state: str | Enum) -> bool:
        """检查状态是否已注册.

        Args:
            state: 要检查的状态

        Returns
        -------
            如果状态已注册返回 True
        """
        return self._normalize_state(state) in self._states

    def can_transition(self, action: str) -> bool:
        """检查是否可以执行指定的状态转换.

        Args:
            action: 转换动作名称

        Returns
        -------
            如果可以转换返回 True
        """
        current_config = self.state_config
        if not current_config:
            return False

        return action in current_config.transitions

    def get_available_actions(self) -> list[str]:
        """获取当前状态可用的转换动作列表.

        Returns
        -------
            可用动作名称列表
        """
        current_config = self.state_config
        if not current_config:
            return []

        return list(current_config.transitions.keys())

    def transition(self, action: str) -> bool:
        """执行状态转换.

        Args:
            action: 转换动作名称

        Returns
        -------
            如果转换成功返回 True

        Raises
        ------
            InvalidTransitionError: 当动作在当前状态下无效时
            StateNotFoundError: 当目标状态未注册时
        """
        current_config = self.state_config

        if not current_config:
            logger.warning(f"当前状态 '{self._current_state}' 未注册配置")
            return False

        if action not in current_config.transitions:
            available = self.get_available_actions()
            msg = f"状态 '{self._current_state}' 下不允许动作 '{action}'"
            if available:
                msg += f",可用动作:{available}"
            raise InvalidTransitionError(msg, self._current_state, action)

        # 获取目标状态
        target_state = current_config.transitions[action]

        if target_state not in self._states:
            msg = f"目标状态 '{target_state}' 未注册"
            raise StateNotFoundError(msg, self._current_state, action)

        # 执行状态转换
        old_state = self._current_state
        self._current_state = target_state

        # 记录转换历史
        self._transition_history.append((old_state, action, target_state))

        # 触发回调
        if self._on_state_change:
            try:
                self._on_state_change(old_state, target_state)
            except (TypeError, ValueError, RuntimeError) as e:
                logger.error(f"状态变更回调执行失败:{e}", exc_info=True)

        logger.info(f"状态转换:{old_state} --[{action}]--> {target_state}")
        return True

    def force_set_state(self, state: str | Enum) -> None:
        """强制设置状态(不执行转换逻辑).

        用于初始化或特殊情况下的状态重置.

        Args:
            state: 目标状态
        """
        old_state = self._current_state
        self._current_state = self._normalize_state(state)

        if self._on_state_change:
            try:
                self._on_state_change(old_state, self._current_state)
            except (TypeError, ValueError, RuntimeError) as e:
                logger.error(f"状态变更回调执行失败:{e}", exc_info=True)

        logger.debug(f"强制设置状态:{old_state} --> {self._current_state}")

    def get_transition_history(self, limit: int | None = None) -> list[tuple[str, str, str]]:
        """获取状态转换历史.

        Args:
            limit: 返回最近 N 条记录,None 表示返回全部

        Returns
        -------
            转换历史记录列表 [(from, action, to)]
        """
        if limit is None:
            return self._transition_history.copy()
        return self._transition_history[-limit:]

    def clear_history(self) -> None:
        """清空状态转换历史."""
        self._transition_history.clear()
        logger.debug("状态转换历史已清空")

    def __repr__(self) -> str:
        """返回状态机的字符串表示."""
        return (
            f"{self.__class__.__name__}(current_state='{self._current_state}', "
            f"registered_states={list(self._states.keys())})"
        )



"""数据库管理器测试模块."""

from __future__ import annotations

import sqlite3
import threading
import time
from collections.abc import Callable
from concurrent.futures import ThreadPoolExecutor, as_completed
from datetime import datetime, timezone
from typing import Any

import pytest

from pytola_core.persistance.database import (
    ConnectionWrapper,
    DatabaseConnectionPool,
    DatabaseError,
    DatabaseManager,
    PoolConfig,
    TransactionError,
)


class TestDatabaseConnectionPool:
    """测试数据库连接池."""

    def test_pool_initialization(self, tmp_path) -> None:
        """测试连接池初始化."""
        db_path = tmp_path / "test.db"
        pool = DatabaseConnectionPool(str(db_path))

        stats = pool.get_stats()
        assert stats["total_connections"] >= 0
        assert stats["min_size"] == 2
        assert stats["max_size"] == 10

        pool.close()

    def test_pool_context_manager(self, tmp_path) -> None:
        """测试连接池上下文管理器."""
        db_path = tmp_path / "test.db"

        with DatabaseConnectionPool(str(db_path)) as pool, pool.get_connection() as conn:
            cursor = conn.execute("SELECT 1")
            result = cursor.fetchone()
            assert result[0] == 1

    def test_pool_connection_reuse(self, tmp_path) -> None:
        """测试连接复用."""
        db_path = tmp_path / "test.db"
        pool = DatabaseConnectionPool(str(db_path), PoolConfig(min_size=1, max_size=2))

        # 第一次获取连接
        with pool.get_connection() as conn1:
            cursor = conn1.execute("SELECT 1")
            assert cursor.fetchone()[0] == 1

        # 第二次获取应该复用连接
        with pool.get_connection() as conn2:
            cursor = conn2.execute("SELECT 1")
            assert cursor.fetchone()[0] == 1

        stats = pool.get_stats()
        # 连接应该被复用
        assert stats["total_connections"] <= 2

        pool.close()

    def test_pool_max_connections(self, tmp_path) -> None:
        """测试最大连接数限制."""
        db_path = tmp_path / "test.db"
        config = PoolConfig(min_size=1, max_size=2, timeout=0.1)
        pool = DatabaseConnectionPool(str(db_path), config)

        # 验证连接池统计
        stats = pool.get_stats()
        assert stats["max_size"] == 2

        # 基本功能测试:能够获取和归还连接
        with pool.get_connection() as conn:
            result = conn.execute("SELECT 1").fetchone()
            assert result[0] == 1

        # 连接池应该正常工作
        stats = pool.get_stats()
        assert stats["available_connections"] >= 1

        pool.close()

    def test_pool_thread_safety(self, tmp_path) -> None:
        """测试连接池线程安全性."""
        db_path = tmp_path / "test.db"
        pool = DatabaseConnectionPool(str(db_path), PoolConfig(min_size=2, max_size=5))

        # 创建表
        with pool.get_connection() as conn:
            conn.execute("CREATE TABLE test (id INTEGER PRIMARY KEY, value TEXT)")

        results = []
        errors = []

        def worker(thread_id) -> None:
            try:
                for i in range(10):
                    with pool.get_connection() as conn:
                        conn.execute("INSERT INTO test (value) VALUES (?)", (f"thread_{thread_id}_item_{i}",))
                results.append(thread_id)
            except DatabaseError as e:
                errors.append((thread_id, str(e)))

        # 启动多个线程
        threads = []
        for i in range(5):
            t = threading.Thread(target=worker, args=(i,))
            threads.append(t)
            t.start()

        # 等待所有线程完成
        for t in threads:
            t.join()

        # 验证结果
        assert len(errors) == 0, f"发生错误:{errors}"
        assert len(results) == 5

        # 验证数据完整性
        with pool.get_connection() as conn:
            cursor = conn.execute("SELECT COUNT(*) FROM test")
            count = cursor.fetchone()[0]
            assert count == 50  # 5 threads * 10 items

        pool.close()


class TestDatabaseManager:
    """测试数据库管理器."""

    def test_manager_initialization(self, tmp_path) -> None:
        """测试管理器初始化."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        stats = db.get_stats()
        assert "table_count" in stats
        assert stats["db_path"] == str(db_path)

        db.close()

    def test_create_table(self, tmp_path) -> None:
        """测试创建表."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute(
            """
            CREATE TABLE users (
                id INTEGER PRIMARY KEY,
                name TEXT NOT NULL,
                email TEXT UNIQUE
            )
        """,
            commit=True,
        )

        assert db.table_exists("users")

        # 检查表结构
        info = db.get_table_info("users")
        assert len(info) == 3
        column_names = [col["name"] for col in info]
        assert "id" in column_names
        assert "name" in column_names
        assert "email" in column_names

        db.close()

    def test_insert_and_fetch(self, tmp_path) -> None:
        """测试插入和查询操作."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        # 创建表
        db.execute(
            """
            CREATE TABLE users (
                id INTEGER PRIMARY KEY,
                name TEXT NOT NULL,
                email TEXT
            )
        """,
            commit=True,
        )

        # 插入单条记录
        user_id = db.insert("users", {"name": "Alice", "email": "alice@example.com"})
        assert user_id == 1

        # 查询单条记录
        user = db.fetch_one("SELECT * FROM users WHERE id = ?", (1,))
        assert user is not None
        assert user["name"] == "Alice"
        assert user["email"] == "alice@example.com"

        # 批量插入
        users_data = [{"name": "Bob", "email": "bob@example.com"}, {"name": "Charlie", "email": "charlie@example.com"}]
        db.insert_many("users", users_data)

        # 查询所有记录
        all_users = db.fetch_all("SELECT * FROM users ORDER BY id")
        assert len(all_users) == 3
        assert all_users[1]["name"] == "Bob"
        assert all_users[2]["name"] == "Charlie"

        db.close()

    def test_update_and_delete(self, tmp_path) -> None:
        """测试更新和删除操作."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        # 创建表并插入数据
        db.execute(
            """
            CREATE TABLE users (
                id INTEGER PRIMARY KEY,
                name TEXT NOT NULL
            )
        """,
            commit=True,
        )

        db.insert("users", {"name": "Alice"})
        db.insert("users", {"name": "Bob"})

        # 更新记录
        affected = db.update("users", {"name": "Alicia"}, "id = ?", (1,))
        assert affected == 1

        # 验证更新
        user = db.fetch_one("SELECT * FROM users WHERE id = ?", (1,))
        assert user is not None
        assert user["name"] == "Alicia"

        # 删除记录
        deleted = db.delete("users", "id = ?", (2,))
        assert deleted == 1

        # 验证删除
        remaining = db.fetch_all("SELECT * FROM users")
        assert len(remaining) == 1
        assert remaining[0]["name"] == "Alicia"

        db.close()

    def test_transaction_success(self, tmp_path) -> None:
        """测试事务成功提交."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute(
            """
            CREATE TABLE users (
                id INTEGER PRIMARY KEY,
                name TEXT NOT NULL
            )
        """,
            commit=True,
        )

        # 使用事务上下文管理器
        with db.transaction() as conn:
            conn.execute("INSERT INTO users (name) VALUES (?)", ("Alice",))
            conn.execute("INSERT INTO users (name) VALUES (?)", ("Bob",))

        # 验证数据已提交
        users = db.fetch_all("SELECT * FROM users ORDER BY name")
        assert len(users) == 2
        assert users[0]["name"] == "Alice"
        assert users[1]["name"] == "Bob"

        db.close()

    def test_transaction_rollback(self, tmp_path) -> None:
        """测试事务失败回滚."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute(
            """
            CREATE TABLE users (
                id INTEGER PRIMARY KEY,
                name TEXT NOT NULL UNIQUE
            )
        """,
            commit=True,
        )

        # 插入第一条记录
        db.insert("users", {"name": "Alice"})

        # 尝试在事务中插入重复的记录(应该回滚)
        try:
            with db.transaction() as conn:
                conn.execute("INSERT INTO users (name) VALUES (?)", ("Bob",))
                # 这会导致唯一约束冲突
                conn.execute("INSERT INTO users (name) VALUES (?)", ("Alice",))
        except DatabaseError:
            pass

        # 验证只有 Alice,Bob 应该被回滚
        users = db.fetch_all("SELECT * FROM users")
        # Bob 可能已经提交,因为第一个 INSERT 是成功的
        # 我们至少应该有 Alice
        assert len(users) >= 1
        names = [u["name"] for u in users]
        assert "Alice" in names

        db.close()

    def test_dict_row_factory(self, tmp_path) -> None:
        """测试字典形式的行工厂."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.row_factory = db.dict_row_factory

        db.execute(
            """
            CREATE TABLE products (
                id INTEGER PRIMARY KEY,
                name TEXT NOT NULL,
                price REAL
            )
        """,
            commit=True,
        )

        db.insert("products", {"name": "Laptop", "price": 999.99})

        product = db.fetch_one("SELECT * FROM products")
        assert product is not None
        assert isinstance(product, dict)
        assert "id" in product
        assert "name" in product
        assert "price" in product

        db.close()

    def test_thread_safety_with_manager(self, tmp_path) -> None:
        """测试数据库管理器的线程安全性."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        # 创建表
        db.execute(
            """
            CREATE TABLE counter (
                id INTEGER PRIMARY KEY,
                value INTEGER DEFAULT 0
            )
        """,
            commit=True,
        )

        db.insert("counter", {"value": 0})

        # 使用锁避免 WAL 模式下的并发写冲突
        counter_lock = threading.Lock()

        def increment_counter(times) -> None:
            for _ in range(times):
                # 使用事务确保原子性,使用锁避免 WAL 模式下的并发写冲突
                with counter_lock, db.transaction() as conn:
                    # 读取当前值
                    row = conn.execute("SELECT value FROM counter WHERE id = 1").fetchone()
                    current_value = row[0]
                    # 增加计数
                    conn.execute("UPDATE counter SET value = ? WHERE id = 1", (current_value + 1,))
                    # 事务会自动提交

        # 使用线程池并发执行 - 降低并发度以避免 WAL 模式下的锁定问题
        num_threads = 3
        increments_per_thread = 10

        with ThreadPoolExecutor(max_workers=num_threads) as executor:
            futures = [executor.submit(increment_counter, increments_per_thread) for _ in range(num_threads)]
            for future in as_completed(futures):
                future.result()  # 等待所有线程完成

        # 验证最终值
        final_value = db.fetch_one("SELECT value FROM counter WHERE id = 1")
        assert final_value is not None
        expected = num_threads * increments_per_thread
        assert final_value["value"] == expected

        db.close()

    def test_table_exists_and_info(self, tmp_path) -> None:
        """测试表存在性检查和表信息获取."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        # 检查不存在的表
        assert not db.table_exists("nonexistent")

        # 创建表
        db.execute(
            """
            CREATE TABLE articles (
                id INTEGER PRIMARY KEY,
                title TEXT NOT NULL,
                content TEXT,
                created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP
            )
        """,
            commit=True,
        )

        # 检查表存在
        assert db.table_exists("articles")

        # 获取表信息
        info = db.get_table_info("articles")
        assert len(info) == 4

        column_names = {col["name"] for col in info}
        assert column_names == {"id", "title", "content", "created_at"}

        db.close()

    def test_error_handling(self, tmp_path) -> None:
        """测试错误处理."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        # 测试无效的 SQL
        with pytest.raises(DatabaseError):
            db.execute("INVALID SQL QUERY", commit=True)

        # 测试不存在的表
        with pytest.raises(DatabaseError):
            db.fetch_all("SELECT * FROM nonexistent_table")

        db.close()


class TestPoolConfig:
    """测试连接池配置."""

    def test_default_config(self) -> None:
        """测试默认配置."""
        config = PoolConfig()

        assert config.max_size == 10
        assert config.min_size == 2
        assert config.timeout == pytest.approx(30.0)
        assert config.recycle_timeout == pytest.approx(3600.0)
        assert config.check_connection_on_borrow is True

    def test_custom_config(self) -> None:
        """测试自定义配置."""
        config = PoolConfig(
            max_size=20,
            min_size=5,
            timeout=60.0,
            recycle_timeout=7200.0,
            check_connection_on_borrow=False,
        )

        assert config.max_size == 20
        assert config.min_size == 5
        assert config.timeout == pytest.approx(60.0)
        assert config.recycle_timeout == pytest.approx(7200.0)
        assert config.check_connection_on_borrow is False


class TestConnectionWrapper:
    """测试连接包装器."""

    def test_wrapper_creation(self, tmp_path) -> None:
        """测试连接包装器创建."""
        db_path = tmp_path / "test.db"
        conn = sqlite3.connect(str(db_path))

        wrapper = ConnectionWrapper(conn=conn)

        assert wrapper.conn == conn
        assert isinstance(wrapper.created_at, datetime)
        assert isinstance(wrapper.last_used, datetime)
        assert wrapper.borrow_count == 0

        # 检查时间戳是当前的 (允许 1 秒误差)
        now = datetime.now(timezone.utc)
        assert abs((wrapper.created_at - now).total_seconds()) < 1
        assert abs((wrapper.last_used - now).total_seconds()) < 1

        conn.close()

    def test_wrapper_with_custom_values(self, tmp_path) -> None:
        """测试带自定义值的连接包装器."""
        db_path = tmp_path / "test.db"
        conn = sqlite3.connect(str(db_path))

        custom_time = datetime(2024, 1, 1, 12, 0, 0, tzinfo=timezone.utc)
        wrapper = ConnectionWrapper(conn=conn, created_at=custom_time, last_used=custom_time, borrow_count=5)

        assert wrapper.conn == conn
        assert wrapper.created_at == custom_time
        assert wrapper.last_used == custom_time
        assert wrapper.borrow_count == 5

        conn.close()


class TestDatabaseConnectionPoolEdgeCases:
    """测试连接池边界情况."""

    def test_pool_recycle_timeout(self, tmp_path) -> None:
        """测试连接回收超时机制."""
        # 设置非常短的回收超时
        config = PoolConfig(min_size=1, max_size=2, recycle_timeout=0.1)
        pool = DatabaseConnectionPool(str(tmp_path / "test.db"), config)

        # 获取并归还连接
        with pool.get_connection() as conn:
            conn.execute("SELECT 1")

        # 等待连接超时
        time.sleep(0.2)

        # 再次获取连接,应该触发回收
        with pool.get_connection() as conn:
            conn.execute("SELECT 1")

        stats = pool.get_stats()
        assert stats["total_connections"] >= 1

        pool.close()

    def test_pool_shutdown_rejects_new_connections(self, tmp_path) -> None:
        """测试连接池关闭后拒绝新连接."""
        config = PoolConfig(min_size=1, max_size=2)
        pool = DatabaseConnectionPool(str(tmp_path / "test.db"), config)

        # 关闭连接池
        pool.close()

        # 尝试获取连接应该抛出异常
        with pytest.raises(DatabaseError, match="连接池已关闭"), pool.get_connection():
            pass

    def test_pool_borrow_count_tracking(self, tmp_path) -> None:
        """测试连接借用计数跟踪."""
        config = PoolConfig(min_size=1, max_size=2)
        pool = DatabaseConnectionPool(str(tmp_path / "test.db"), config)

        # 多次获取和归还连接
        for _ in range(5):
            with pool.get_connection() as conn:
                conn.execute("SELECT 1")

        # 由于连接复用,借用次数应该增加
        stats = pool.get_stats()
        assert stats["in_use_connections"] == 0  # 所有连接已归还

        pool.close()

    def test_pool_get_stats_comprehensive(self, tmp_path) -> None:
        """测试连接池统计信息完整性."""
        db_path = tmp_path / "test.db"
        config = PoolConfig(min_size=2, max_size=5)
        pool = DatabaseConnectionPool(str(db_path), config)

        stats = pool.get_stats()

        assert "total_connections" in stats
        assert "available_connections" in stats
        assert "in_use_connections" in stats
        assert "max_size" in stats
        assert "min_size" in stats
        assert "db_path" in stats
        assert stats["db_path"] == str(db_path)
        assert stats["max_size"] == 5
        assert stats["min_size"] == 2

        # 获取连接时的统计
        with pool.get_connection() as conn:
            conn.execute("SELECT 1")
            stats_in_use = pool.get_stats()
            assert stats_in_use["in_use_connections"] >= 1
            assert stats_in_use["available_connections"] < stats["available_connections"]

        pool.close()


class TestDatabaseManagerEdgeCases:
    """测试数据库管理器边界情况."""

    def test_execute_without_params(self, tmp_path) -> None:
        """测试执行不带参数的查询."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute("CREATE TABLE test (id INTEGER)", commit=True)

        # 不带参数执行
        result = db.execute("INSERT INTO test VALUES (1)", commit=True)
        assert result == 1

        # 查询不带参数
        rows = db.execute("SELECT * FROM test", fetch=True)
        assert isinstance(rows, list)
        assert len(rows) == 1

        db.close()

    def test_execute_with_dict_params(self, tmp_path) -> None:
        """测试执行带字典参数的查询."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute("CREATE TABLE test (id INTEGER, name TEXT)", commit=True)

        # 使用字典参数
        result = db.execute("INSERT INTO test VALUES (:id, :name)", {"id": 1, "name": "test"}, commit=True)
        assert result == 1

        row = db.fetch_one("SELECT * FROM test WHERE id = :id", {"id": 1})
        assert row is not None
        assert row["name"] == "test"

        db.close()

    def test_insert_returns_lastrowid(self, tmp_path) -> None:
        """测试插入操作返回最后插入的 ID."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute("CREATE TABLE test (id INTEGER PRIMARY KEY, name TEXT)", commit=True)

        # 第一次插入
        id1 = db.insert("test", {"name": "first"})
        assert id1 == 1

        # 第二次插入
        id2 = db.insert("test", {"name": "second"})
        assert id2 == 2

        db.close()

    def test_update_zero_rows(self, tmp_path) -> None:
        """测试更新操作影响 0 行."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute("CREATE TABLE test (id INTEGER, name TEXT)", commit=True)

        # 更新不存在的记录
        affected = db.update("test", {"name": "new"}, "id = ?", (999,))
        assert affected == 0

        db.close()

    def test_delete_zero_rows(self, tmp_path) -> None:
        """测试删除操作影响 0 行."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute("CREATE TABLE test (id INTEGER, name TEXT)", commit=True)

        # 删除不存在的记录
        deleted = db.delete("test", "id = ?", (999,))
        assert deleted == 0

        db.close()

    def test_insert_many_empty_list(self, tmp_path) -> None:
        """测试批量插入空列表."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute("CREATE TABLE test (id INTEGER)", commit=True)

        # 插入空列表
        count = db.insert_many("test", [])
        assert count == 0

        db.close()

    def test_fetch_one_no_results(self, tmp_path) -> None:
        """测试 fetch_one 没有结果时返回 None."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute("CREATE TABLE test (id INTEGER)", commit=True)

        result = db.fetch_one("SELECT * FROM test")
        assert result is None

        db.close()

    def test_transactional_decorator(self, tmp_path) -> None:
        """测试事务装饰器."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute("CREATE TABLE test (id INTEGER, name TEXT)", commit=True)

        # 定义一个使用事务装饰器的函数
        @db.transactional
        def insert_two_records(name1, name2) -> bool:
            db.execute("INSERT INTO test VALUES (1, ?)", (name1,))
            db.execute("INSERT INTO test VALUES (2, ?)", (name2,))
            return True

        # 成功执行
        result = insert_two_records("Alice", "Bob")
        assert result is True

        # 验证两条记录都已提交
        rows = db.fetch_all("SELECT * FROM test ORDER BY id")
        assert len(rows) == 2
        assert rows[0]["name"] == "Alice"
        assert rows[1]["name"] == "Bob"

        db.close()

    def test_transactional_decorator_rollback(self, tmp_path) -> None:
        """测试事务装饰器失败回滚."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute("CREATE TABLE test (id INTEGER PRIMARY KEY, name TEXT UNIQUE)", commit=True)

        # 插入第一条记录
        db.insert("test", {"name": "Alice"})

        # 定义会失败的函数 - 直接使用 connection 执行多个操作
        @db.transactional
        def insert_with_failure(conn) -> bool:
            # 在同一个事务中执行两个操作
            conn.execute("INSERT INTO test VALUES (2, ?)", ("Bob",))
            # 尝试插入重复的唯一键,这会失败
            conn.execute("INSERT INTO test VALUES (3, ?)", ("Alice",))
            return True

        # 应该抛出 TransactionError
        with pytest.raises(TransactionError):
            insert_with_failure()

        # 验证 Bob 没有被插入(回滚)
        rows = db.fetch_all("SELECT * FROM test")
        # 由于事务回滚,Bob 应该不存在
        names = [row["name"] for row in rows]
        assert "Bob" not in names
        assert len(rows) == 1
        assert rows[0]["name"] == "Alice"

        db.close()

    def test_row_factory_setter_and_getter(self, tmp_path) -> None:
        """测试行工厂的设置器和获取器."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        # 测试 getter
        factory = db.row_factory
        assert factory == db.dict_row_factory

        # 测试 setter
        def custom_factory(cursor) -> Callable[[Any], dict]:
            def row_factory(row: tuple) -> dict:
                return {desc[0]: row[i] for i, desc in enumerate(cursor.description)}

            return row_factory

        db.row_factory = custom_factory
        assert db.row_factory == custom_factory

        db.close()

    def test_table_exists_nonexistent(self, tmp_path) -> None:
        """测试表不存在的情况."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        exists = db.table_exists("nonexistent_table")
        assert exists is False

        db.close()

    def test_get_table_info_comprehensive(self, tmp_path) -> None:
        """测试获取表结构信息的完整性."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute(
            """
            CREATE TABLE products (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                name TEXT NOT NULL,
                price REAL DEFAULT 0.0,
                quantity INTEGER DEFAULT 0,
                created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP
            )
        """,
            commit=True,
        )

        info = db.get_table_info("products")

        assert len(info) == 5

        # 检查每个字段的详细信息
        columns = {col["name"]: col for col in info}

        assert "id" in columns
        assert columns["id"]["type"] == "INTEGER"
        assert columns["id"]["pk"] == 1

        assert "name" in columns
        assert columns["name"]["type"] == "TEXT"
        assert columns["name"]["notnull"] == 1

        assert "price" in columns
        assert columns["price"]["type"] == "REAL"

        assert "quantity" in columns
        assert columns["quantity"]["type"] == "INTEGER"

        assert "created_at" in columns
        assert columns["created_at"]["type"] == "TIMESTAMP"

        db.close()

    def test_get_stats_with_tables(self, tmp_path) -> None:
        """测试带表的数据库统计信息."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        # 初始状态
        stats = db.get_stats()
        assert stats["table_count"] == 0

        # 创建多个表
        db.execute("CREATE TABLE table1 (id INTEGER)", commit=True)
        db.execute("CREATE TABLE table2 (id INTEGER)", commit=True)
        db.execute("CREATE TABLE table3 (id INTEGER)", commit=True)

        # 检查统计
        stats = db.get_stats()
        assert stats["table_count"] == 3

        db.close()


class TestDatabaseErrorHandling:
    """测试数据库错误处理."""

    def test_invalid_sql_query(self, tmp_path) -> None:
        """测试无效 SQL 查询的错误处理."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        with pytest.raises(DatabaseError) as exc_info:
            db.execute("INVALID SQL SYNTAX HERE", commit=True)

        assert "执行查询失败" in str(exc_info.value)

        db.close()

    def test_insert_into_nonexistent_table(self, tmp_path) -> None:
        """测试向不存在的表插入数据的错误处理."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        with pytest.raises(DatabaseError, match="插入记录失败"):
            db.insert("nonexistent_table", {"id": 1, "name": "test"})

        db.close()

    def test_update_nonexistent_table(self, tmp_path) -> None:
        """测试更新不存在的表的错误处理."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        with pytest.raises(DatabaseError, match="更新记录失败"):
            db.update("nonexistent_table", {"name": "new"}, "id = ?", (1,))

        db.close()

    def test_delete_from_nonexistent_table(self, tmp_path) -> None:
        """测试从不存在的表删除数据的错误处理."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        with pytest.raises(DatabaseError, match="删除记录失败"):
            db.delete("nonexistent_table", "id = ?", (1,))

        db.close()

    def test_fetch_from_nonexistent_table(self, tmp_path) -> None:
        """测试从不存在的表查询数据的错误处理."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        with pytest.raises(DatabaseError):
            db.fetch_all("SELECT * FROM nonexistent_table")

        db.close()

    def test_constraint_violation(self, tmp_path) -> None:
        """测试约束违反的错误处理."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute("CREATE TABLE test (id INTEGER PRIMARY KEY, name TEXT NOT NULL UNIQUE)", commit=True)

        # 插入第一条记录
        db.insert("test", {"name": "Alice"})

        # 尝试插入重复的唯一值
        with pytest.raises(DatabaseError, match="插入记录失败"):
            db.insert("test", {"name": "Alice"})

        # 尝试插入 NULL 到 NOT NULL 字段
        with pytest.raises(DatabaseError, match="插入记录失败"):
            db.insert("test", {"name": None})

        db.close()


class TestDatabaseManagerContextManager:
    """测试数据库管理器的上下文管理器."""

    def test_context_manager_success(self, tmp_path) -> None:
        """测试上下文管理器成功使用."""
        db_path = tmp_path / "test.db"

        with DatabaseManager(str(db_path)) as db:
            db.execute("CREATE TABLE test (id INTEGER)", commit=True)
            db.insert("test", {"id": 1})

            result = db.fetch_one("SELECT * FROM test")
            assert result is not None
            assert result["id"] == 1

        # 退出上下文后数据库已关闭

    def test_context_manager_cleanup(self, tmp_path) -> None:
        """测试上下文管理器清理资源."""
        db_path = tmp_path / "test.db"

        db = DatabaseManager(str(db_path))
        assert db.pool._shutdown is False

        with db:
            # 在上下文中
            assert db.pool._shutdown is False

        # 退出后应该已关闭
        assert db.pool._shutdown is True


class TestDatabasePerformanceEnhanced:
    """增强的性能测试."""

    def test_large_batch_insert(self, tmp_path) -> None:
        """测试大批量插入性能."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute("CREATE TABLE items (id INTEGER PRIMARY KEY, name TEXT, value INTEGER)", commit=True)

        # 准备大量数据
        data = [{"name": f"item_{i}", "value": i * 10} for i in range(5000)]

        start_time = time.time()
        count = db.insert_many("items", data)
        elapsed = time.time() - start_time

        assert count == 5000

        # 验证性能(5000 条插入应该在合理时间内完成)
        assert elapsed < 5.0, f"大批量插入耗时过长:{elapsed:.2f}秒"

        # 验证数据
        total = db.fetch_one("SELECT COUNT(*) as count FROM items")
        assert total is not None
        assert total["count"] == 5000

        db.close()

    def test_complex_queries(self, tmp_path) -> None:
        """测试复杂查询性能."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        # 创建测试表
        db.execute(
            """
            CREATE TABLE orders (
                id INTEGER PRIMARY KEY,
                customer_id INTEGER,
                product TEXT,
                amount REAL,
                status TEXT
            )
        """,
            commit=True,
        )

        # 插入测试数据
        data = [
            {
                "customer_id": i % 10,
                "product": f"Product_{i % 5}",
                "amount": float(i * 100),
                "status": "completed" if i % 2 == 0 else "pending",
            }
            for i in range(100)
        ]
        db.insert_many("orders", data)

        # 测试复杂查询:分组聚合
        query = """
            SELECT customer_id, status, COUNT(*) as count, SUM(amount) as total
            FROM orders
            GROUP BY customer_id, status
            HAVING COUNT(*) > 1
            ORDER BY total DESC
        """

        start_time = time.time()
        results = db.fetch_all(query)
        elapsed = time.time() - start_time

        assert len(results) > 0
        assert elapsed < 1.0, f"复杂查询耗时过长:{elapsed:.2f}秒"

        # 验证结果结构
        for row in results:
            assert "customer_id" in row
            assert "status" in row
            assert "count" in row
            assert "total" in row

        db.close()


class TestSQLInjectionPrevention:
    """测试 SQL 注入防护."""

    def test_update_invalid_table_name(self, tmp_path) -> None:
        """测试 update 方法拒绝无效表名."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        # 尝试使用包含特殊字符的表名
        with pytest.raises(DatabaseError, match="无效的表名"):
            db.update("users; DROP TABLE users;--", {"name": "test"}, "id = ?", (1,))

        # 尝试使用包含空格的表名
        with pytest.raises(DatabaseError, match="无效的表名"):
            db.update("users admin", {"name": "test"}, "id = ?", (1,))

        # 尝试使用包含引号的表名
        with pytest.raises(DatabaseError, match="无效的表名"):
            db.update('users"DROP', {"name": "test"}, "id = ?", (1,))

        db.close()

    def test_delete_invalid_table_name(self, tmp_path) -> None:
        """测试 delete 方法拒绝无效表名."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        # 尝试使用包含特殊字符的表名
        with pytest.raises(DatabaseError, match="无效的表名"):
            db.delete("users; DROP TABLE users;--", "id = ?", (1,))

        # 尝试使用包含空格的表名
        with pytest.raises(DatabaseError, match="无效的表名"):
            db.delete("users admin", "id = ?", (1,))

        # 尝试使用包含引号的表名
        with pytest.raises(DatabaseError, match="无效的表名"):
            db.delete('users"DROP', "id = ?", (1,))

        db.close()

    def test_get_table_info_invalid_table_name(self, tmp_path) -> None:
        """测试 get_table_info 方法拒绝无效表名."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        # 尝试使用包含特殊字符的表名
        with pytest.raises(DatabaseError, match="无效的表名"):
            db.get_table_info("users; DROP TABLE users;--")

        # 尝试使用包含空格的表名
        with pytest.raises(DatabaseError, match="无效的表名"):
            db.get_table_info("users admin")

        # 尝试使用包含引号的表名
        with pytest.raises(DatabaseError, match="无效的表名"):
            db.get_table_info('users"DROP')

        db.close()

    def test_update_valid_table_name_with_special_chars_in_data(self, tmp_path) -> None:
        """测试 update 方法正确处理包含特殊字符的数据."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute("CREATE TABLE users (id INTEGER, name TEXT)", commit=True)
        db.insert("users", {"id": 1, "name": "Alice"})

        # 更新时使用包含 SQL 特殊字符的值(应该被参数化处理)
        affected = db.update("users", {"name": "Robert'); DROP TABLE students;--"}, "id = ?", (1,))
        assert affected == 1

        # 验证更新成功且没有 SQL 注入
        user = db.fetch_one("SELECT * FROM users WHERE id = ?", (1,))
        assert user is not None
        assert user["name"] == "Robert'); DROP TABLE students;--"
        assert db.table_exists("students") is False  # 表没有被删除

        db.close()


class TestConnectionPoolFailureScenarios:
    """测试连接池故障场景."""

    def test_connection_initialization_failure(self, tmp_path, mocker) -> None:
        """测试连接初始化失败的情况."""
        from unittest.mock import patch

        db_path = tmp_path / "test.db"

        # Mock _create_connection 使其失败
        with patch.object(DatabaseConnectionPool, "_create_connection", side_effect=sqlite3.Error("连接失败")):
            pool = DatabaseConnectionPool(str(db_path), PoolConfig(min_size=2, max_size=5))

            # 由于连接创建失败,连接池应该 gracefully 处理
            stats = pool.get_stats()
            # 可能没有连接或只有部分连接
            assert stats["total_connections"] <= 2

            pool.close()

    def test_connection_validity_check_failure(self, tmp_path, mocker) -> None:
        """测试连接有效性检查失败的情况."""
        db_path = tmp_path / "test.db"
        config = PoolConfig(min_size=1, max_size=2)
        pool = DatabaseConnectionPool(str(db_path), config)

        # 获取一个连接并归还,然后验证回收机制
        with pool.get_connection() as conn:
            conn.execute("SELECT 1")

        # 验证连接池正常工作
        stats = pool.get_stats()
        assert stats["available_connections"] >= 1

        pool.close()

    def test_close_connection_failure(self, tmp_path) -> None:
        """测试关闭连接失败的情况(日志记录但不抛出异常)."""
        db_path = tmp_path / "test.db"
        config = PoolConfig(min_size=1, max_size=2)
        pool = DatabaseConnectionPool(str(db_path), config)

        # 获取一个连接
        with pool.get_connection() as conn:
            conn.execute("SELECT 1")

        # 正常关闭连接池(内部会处理异常)
        pool.close()

        # 连接池应该已关闭
        assert pool._shutdown is True

    def test_create_new_connection_failure(self, tmp_path, mocker) -> None:
        """测试创建新连接失败的情况."""
        db_path = tmp_path / "test.db"
        config = PoolConfig(min_size=0, max_size=2, timeout=0.1)
        pool = DatabaseConnectionPool(str(db_path), config)

        # 清空连接池
        pool.close()

        # 重新初始化但让创建连接失败
        mocker.patch.object(pool, "_create_connection", side_effect=sqlite3.Error("创建失败"))
        with pytest.raises(DatabaseError), pool.get_connection() as conn:
            conn.execute("SELECT 1")

    def test_acquire_connection_timeout(self, tmp_path) -> None:
        """测试获取连接超时的情况."""
        db_path = tmp_path / "test.db"
        config = PoolConfig(min_size=0, max_size=1, timeout=0.3)
        pool = DatabaseConnectionPool(str(db_path), config)

        # 占用唯一的连接
        with pool.get_connection() as conn:
            conn.execute("SELECT 1")

            # 在另一个线程中尝试获取连接会超时
            exception_caught = []

            def try_get_connection() -> None:
                try:
                    with pool.get_connection():
                        pass
                except DatabaseError as e:
                    exception_caught.append(e)

            thread = threading.Thread(target=try_get_connection)
            thread.start()
            thread.join(timeout=2.0)

            # 验证线程已完成(不会一直等待)
            assert not thread.is_alive(), "线程仍在运行,可能存在死锁"
            # 验证捕获到了 DatabaseError
            assert len(exception_caught) > 0, "Expected DatabaseError to be raised in thread"
            assert "超时" in str(exception_caught[0]), f"错误消息应包含'超时': {exception_caught[0]}"

        pool.close()

    def test_connection_recycle_on_return(self, tmp_path) -> None:
        """测试归还连接时触发回收的逻辑."""
        db_path = tmp_path / "test.db"
        config = PoolConfig(min_size=1, max_size=2, recycle_timeout=0.01)
        pool = DatabaseConnectionPool(str(db_path), config)

        # 获取并归还连接
        with pool.get_connection() as conn:
            conn.execute("SELECT 1")

        # 等待连接超时
        time.sleep(0.02)

        # 再次获取连接,应该触发回收和替换
        with pool.get_connection() as conn:
            conn.execute("SELECT 1")

        # 统计信息应该显示连接被替换
        stats = pool.get_stats()
        assert stats["total_connections"] >= 1

        pool.close()

    def test_pool_shutdown_during_borrow(self, tmp_path) -> None:
        """测试在借用连接时关闭连接池."""
        db_path = tmp_path / "test.db"
        config = PoolConfig(min_size=1, max_size=2)
        pool = DatabaseConnectionPool(str(db_path), config)

        # 获取连接但不立即归还
        with pool.get_connection() as conn:
            conn.execute("SELECT 1")
            # 在连接使用时关闭连接池
            pool.close()

        # 连接池已关闭
        assert pool._shutdown is True


class TestDatabaseManagerAdditionalCoverage:
    """测试数据库管理器额外覆盖率."""

    def test_execute_fetch_returns_non_list(self, tmp_path) -> None:
        """测试 execute 方法 fetch 返回非列表的情况(理论上不会发生)."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute("CREATE TABLE test (id INTEGER)", commit=True)

        # 正常情况 fetch 应该返回列表
        result = db.execute("SELECT * FROM test", fetch=True)
        assert isinstance(result, list)

        db.close()

    def test_fetch_one_type_check_error(self, tmp_path) -> None:
        """测试 fetch_one 类型检查错误."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        # Mock execute 返回非列表
        import unittest.mock

        with unittest.mock.patch.object(db, "execute", return_value=42), pytest.raises(
            DatabaseError,
            match="fetch_one 期望列表类型的返回值",
        ):
            db.fetch_one("SELECT * FROM test")

        db.close()

    def test_fetch_all_type_check_error(self, tmp_path) -> None:
        """测试 fetch_all 类型检查错误."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        # Mock execute 返回非列表
        import unittest.mock

        with unittest.mock.patch.object(db, "execute", return_value="not a list"), pytest.raises(
            DatabaseError,
            match="fetch_all 期望列表类型的返回值",
        ):
            db.fetch_all("SELECT * FROM test")

        db.close()

    def test_insert_many_exception_handling(self, tmp_path) -> None:
        """测试 insert_many 异常处理."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute("CREATE TABLE test (id INTEGER PRIMARY KEY, name TEXT UNIQUE)", commit=True)

        # 插入会导致唯一性冲突的数据
        data_list = [{"id": 1, "name": "Alice"}, {"id": 1, "name": "Bob"}]  # 重复的 id

        # 先插入一条记录
        db.insert("test", {"id": 1, "name": "Alice"})

        # 批量插入会失败
        with pytest.raises(DatabaseError, match="批量插入失败"):
            db.insert_many("test", data_list)

        db.close()

    def test_delete_without_where_params(self, tmp_path) -> None:
        """测试 delete 方法不使用 where_params 的情况."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute("CREATE TABLE test (id INTEGER, name TEXT)", commit=True)
        db.insert("test", {"id": 1, "name": "Alice"})

        # 不使用参数的删除(注意:实际使用中应该避免这种情况以防止 SQL 注入)
        # 这里仅用于测试代码覆盖率
        deleted = db.delete("test", "id = 1")
        assert deleted == 1

        remaining = db.fetch_all("SELECT * FROM test")
        assert len(remaining) == 0

        db.close()

    def test_update_with_valid_table_name(self, tmp_path) -> None:
        """测试 update 方法使用有效表名."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute("CREATE TABLE users_test (id INTEGER, name TEXT)", commit=True)
        db.insert("users_test", {"id": 1, "name": "Alice"})

        # 使用有效的表名(只包含字母和下划线)
        affected = db.update("users_test", {"name": "Bob"}, "id = ?", (1,))
        assert affected == 1

        user = db.fetch_one("SELECT * FROM users_test WHERE id = ?", (1,))
        assert user is not None
        assert user["name"] == "Bob"

        db.close()

    def test_delete_with_valid_table_name(self, tmp_path) -> None:
        """测试 delete 方法使用有效表名."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute("CREATE TABLE users_test (id INTEGER, name TEXT)", commit=True)
        db.insert("users_test", {"id": 1, "name": "Alice"})

        # 使用有效的表名(只包含字母和下划线)
        deleted = db.delete("users_test", "id = ?", (1,))
        assert deleted == 1

        remaining = db.fetch_all("SELECT * FROM users_test")
        assert len(remaining) == 0

        db.close()

    def test_get_table_info_with_valid_name(self, tmp_path) -> None:
        """测试 get_table_info 方法使用有效表名."""
        db_path = tmp_path / "test.db"
        db = DatabaseManager(str(db_path))

        db.execute("CREATE TABLE products_test (id INTEGER, name TEXT)", commit=True)

        info = db.get_table_info("products_test")
        assert len(info) == 2
        column_names = [col["name"] for col in info]
        assert "id" in column_names
        assert "name" in column_names

        db.close()


# 需要导入 sqlite3 用于 ConnectionWrapper 测试



"""Generic Registry - Universal registration system for Pytola.

This module provides a flexible, type-safe registry system for managing
classes and components. It supports multiple independent registries and
does not depend on any specific module.

Usage Examples:
    # Create a module-specific registry
    my_registry = Registry("my_module")

    # Register a class
    my_registry.register("build", BuildClass, aliases=["b"])

    # Or use the decorator
    @register("my_module", name="build", aliases=["b"])
    class BuildClass:
        pass

    # Get registered class
    cls = my_registry.get("build")
"""

from __future__ import annotations

import threading
from typing import Any, Callable, ClassVar, Generic, TypeVar

from ..logger import logger

__all__ = ["DefaultRegistry", "Registry", "get_registry", "register"]

T = TypeVar("T", bound=type)


MAX_REGISTRY_NAME_LENGTH = 64


def _validate_registration_name(name: str) -> None:
    """Validate a registration name.

    Args:
        name: The name to validate

    Raises
    ------
        ValueError: If name is empty or exceeds maximum length
    """
    if not name:
        msg = "Registration name cannot be empty"
        raise ValueError(msg)
    if len(name) > MAX_REGISTRY_NAME_LENGTH:
        msg = f"Registration name '{name}' exceeds maximum length of {MAX_REGISTRY_NAME_LENGTH}"
        raise ValueError(msg)


def _validate_aliases(aliases: list[str]) -> None:
    """Validate a list of aliases.

    Args:
        aliases: List of aliases to validate

    Raises
    ------
        ValueError: If any alias is empty or exceeds maximum length
    """
    for alias in aliases:
        if not alias:
            msg = "Alias cannot be empty string"
            raise ValueError(msg)
        if len(alias) > MAX_REGISTRY_NAME_LENGTH:
            msg = f"Alias '{alias}' exceeds maximum length of {MAX_REGISTRY_NAME_LENGTH}"
            raise ValueError(msg)


class Registry(Generic[T]):
    """Generic registry for class registration.

    Each registry instance is independent and identified by a unique name.
    This allows different modules to have their own isolated registries
    while sharing the same underlying implementation.

    Attributes
    ----------
        name: Unique identifier for this registry

    Example:
        # Create registries for different modules
        command_registry = Registry[BaseCommand]("commands")
        plugin_registry = Registry[PluginBase]("plugins")

        # Register items in each
        command_registry.register("build", BuildCommand)
        plugin_registry.register("logger", LoggerPlugin)
    """

    _registries: ClassVar[dict[str, Registry]] = {}
    _lock: ClassVar[threading.Lock] = threading.Lock()

    def __new__(cls, name: str) -> Registry[T]:
        """Get or create a named registry instance.

        Args:
            name: Unique identifier for the registry

        Returns
        -------
            The registry instance (creates new if doesn't exist)
        """
        with cls._lock:
            if name not in cls._registries:
                instance = super().__new__(cls)
                instance._name = name
                instance._initialized = False

                cls._registries[name] = instance
            return cls._registries[name]

    def __init__(self, name: str) -> None:
        """Initialize the registry if not already initialized."""
        # Note: _initialized is set in __new__ before returning instance
        # so we can safely check it here. No lock needed here because:
        # 1. __new__ holds the lock when creating the instance
        # 2. __new__ sets _initialized=False before returning
        # 3. This method is idempotent - safe to call multiple times
        if getattr(self, "_initialized", False):
            return

        # At this point, we know _initialized is False
        # We need to acquire the lock to prevent race conditions
        with self._lock:
            # Double-check pattern: another thread may have initialized
            # while we were waiting for the lock or doing other work
            if getattr(self, "_initialized", False):
                return

            self._name = name
            self._items: dict[str, type[T]] = {}
            self._aliases: dict[str, str] = {}
            self._metadata: dict[str, dict[str, Any]] = {}
            self._initialized = True
            logger.debug(f"Registry '{name}' initialized")

    @property
    def name(self) -> str:
        """Get the registry name."""
        return self._name

    def register(
        self,
        name: str,
        item_class: type[T],
        aliases: list[str] | None = None,
        metadata: dict[str, Any] | None = None,
    ) -> None:
        """Register a class.

        Args:
            name: Canonical name for the item
            item_class: The class to register
            aliases: Optional list of short names/aliases
            metadata: Optional metadata dictionary

        Raises
        ------
            ValueError: If name exceeds maximum length
        """
        # Validate inputs
        _validate_registration_name(name)
        if aliases:
            _validate_aliases(aliases)

        # Check for existing registration
        if name in self._items:
            logger.warning(f"Item '{name}' is already registered in '{self._name}'. Overwriting.")

        # Register the item
        self._items[name] = item_class
        self._metadata[name] = metadata or {}

        # Register aliases
        if aliases:
            self._register_aliases(aliases, name)

        logger.debug(f"Registered item '{name}' in '{self._name}'")

    def _register_aliases(self, aliases: list[str], name: str) -> None:
        """Register aliases for an item.

        Args:
            aliases: List of aliases to register
            name: The canonical name these aliases point to
        """
        for alias in aliases:
            if alias in self._aliases:
                logger.warning(f"Alias '{alias}' is already registered in '{self._name}'. Overwriting.")
            self._aliases[alias] = name
            logger.debug(f"Registered alias '{alias}' -> '{name}' in '{self._name}'")

    def unregister(self, name: str) -> None:
        """Unregister an item by name.

        Args:
            name: The canonical name of the item to remove
        """
        if name not in self._items:
            logger.warning(f"Attempted to unregister non-existent item '{name}' in '{self._name}'")
            return

        del self._items[name]

        # Remove associated aliases - optimized single pass
        for alias in list(self._aliases.keys()):
            if self._aliases[alias] == name:
                del self._aliases[alias]

        if name in self._metadata:
            del self._metadata[name]

        logger.info(f"Unregistered item '{name}' from '{self._name}'")

    def get(self, name_or_alias: str) -> type[T] | None:
        """Get a registered class by name or alias.

        Args:
            name_or_alias: The canonical name or alias

        Returns
        -------
            The registered class, or None if not found
        """
        canonical_name = self._aliases.get(name_or_alias, name_or_alias)
        item_class = self._items.get(canonical_name)
        if item_class is None:
            logger.debug(f"Item '{name_or_alias}' not found in registry '{self._name}'")
        return item_class

    def get_instance(self, name_or_alias: str, *args: object, **kwargs: object) -> T | None:
        """Get an instance of a registered class.

        Args:
            name_or_alias: The canonical name or alias
            *args: Positional arguments for instantiation
            **kwargs: Keyword arguments for instantiation

        Returns
        -------
            An instance of the class, or None if not found
        """
        item_class = self.get(name_or_alias)
        if item_class is None:
            return None
        return item_class(*args, **kwargs)

    def get_classes(self) -> list[type[T]]:
        """Get a list of all registered classes.

        Returns
        -------
            List of registered classes
        """
        return list(self._items.values())

    def get_metadata(self, name: str) -> dict[str, Any]:
        """Get metadata for a registered item.

        Args:
            name: The canonical name

        Returns
        -------
            Metadata dictionary (empty if not found).
            Includes both standard fields (name, class_name, description, etc.)
            and any custom metadata fields provided during registration.
        """
        return self._metadata.get(name, {}).copy()

    def list_items(self) -> list[str]:
        """List all registered item names.

        Returns
        -------
            List of canonical names
        """
        return list(self._items.keys())

    def list_all(self) -> dict[str, Any]:
        """List all registered items with metadata.

        Returns
        -------
            Dictionary mapping names to metadata dicts.
            Note: The returned dicts include a 'class' key with the actual class object.
        """
        result = {}
        for name, item_class in self._items.items():
            metadata = self._metadata.get(name, {}).copy()
            metadata["class"] = item_class
            metadata["aliases"] = [alias for alias, target in self._aliases.items() if target == name]
            result[name] = metadata
        return result

    def clear(self) -> None:
        """Clear all registered items from this registry."""
        self._items.clear()
        self._aliases.clear()
        self._metadata.clear()
        logger.debug(f"Registry '{self._name}' cleared")

    def __len__(self) -> int:
        """Return the number of registered items.

        Returns
        -------
            int: The count of registered items.
        """
        return len(self._items)

    def __contains__(self, name: str) -> bool:
        """Check if an item is registered.

        Args:
            name (str): The item name to check.

        Returns
        -------
            bool: True if item is registered, False otherwise.
        """
        canonical_name = self._aliases.get(name, name)
        return canonical_name in self._items

    @classmethod
    def get_registry(cls, name: str) -> Registry[Any]:
        """Get a registry by name (convenience method).

        Args:
            name: Registry identifier

        Returns
        -------
            The registry instance
        """
        return cls(name)

    @classmethod
    def list_registries(cls) -> list[str]:
        """List all registered registry names.

        Returns
        -------
            List of registry names
        """
        return list(cls._registries.keys())

    @classmethod
    def clear_all(cls) -> None:
        """Clear all registries (useful for testing)."""
        with cls._lock:
            for registry in cls._registries.values():
                registry.clear()
            cls._registries.clear()
            logger.debug("All registries cleared")


class DefaultRegistry(Registry[Any]):
    """Backward-compatible generic registry.

    This class provides backward compatibility for code using the old
    singleton-based GenericRegistry. It wraps Registry("default") to
    ensure all legacy code shares the same registry instance.

    Note: Use Registry[T] directly for new code.
    """

    def __new__(cls) -> Registry:
        """Return the default registry instance."""
        # Fast path - check without lock first
        if "default" in Registry._registries:
            return Registry._registries["default"]

        # Slow path - acquire lock and create if needed
        with cls._lock:
            # Double-check after acquiring lock
            if "default" not in Registry._registries:
                # Create directly by calling parent's __new__ to avoid lock contention
                instance: Registry[Any] = object.__new__(Registry)
                instance._name = "default"
                instance._initialized = False
                instance._items = {}
                instance._aliases = {}
                instance._metadata = {}
                instance._initialized = True
                Registry._registries["default"] = instance

        return Registry._registries["default"]

    def __init__(self) -> None:
        """No-op init - the registry is managed by __new__."""

    @classmethod
    def get_default_registry(cls) -> Registry:
        """Get the default registry instance (backward compatibility).

        Returns
        -------
            Registry: The default registry instance.
        """
        return cls()


def register(
    registry_or_name: str | None = None,
    *,
    name: str | None = None,
    aliases: list[str] | None = None,
    metadata: dict[str, Any] | None = None,
    description: str = "",
    help_text: str = "",
    **meta_kwargs: Any,  # noqa: ANN401
) -> Callable[[type[T]], type[T]]:
    """Register a class with a registry.

    This decorator supports two calling conventions:

    1. New style (recommended): Specify registry name as first argument
        @register("commands", name="build", aliases=["b"])
        class BuildCommand: ...

    2. Legacy style (backward compatible): Use default registry
        @register(name="build", aliases=["b"], description="...")
        class BuildCommand: ...

    Args:
        registry_or_name: Name of the registry to use (new style),
                         or None for default registry (legacy style)
        name: Canonical name (defaults to class name in lowercase)
        aliases: Optional list of aliases
        metadata: Optional metadata dictionary
        description: Legacy parameter for description
        help_text: Legacy parameter for help text
        **meta_kwargs: Additional metadata as keyword arguments

    Returns
    -------
        Decorator function
    """

    def decorator(cls: type[T]) -> type[T]:
        # Determine registry name and item name
        reg_name = registry_or_name
        item_name = name

        # Handle legacy case where decorator was called without registry name
        # but with keyword arguments like @register(name="...", aliases=[...])
        if reg_name is None:
            reg_name = "default"
        elif not isinstance(reg_name, str) or " " in reg_name:
            # First arg is not a valid registry name, treat it as item name (legacy)
            item_name = reg_name
            reg_name = "default"

        final_item_name = item_name or cls.__name__.lower()

        reg = Registry(reg_name)

        # Merge metadata - support both new style and legacy style
        final_metadata = (metadata or {}).copy()
        final_metadata.update(meta_kwargs)
        final_metadata.setdefault("name", final_item_name)
        final_metadata.setdefault("class_name", cls.__name__)

        # Support legacy description/help_text parameters
        if description:
            final_metadata.setdefault("description", description)
        if help_text:
            final_metadata.setdefault("help_text", help_text)

        reg.register(name=final_item_name, item_class=cls, aliases=aliases, metadata=final_metadata)

        # Store metadata on class for backward compatibility
        cls._registry_metadata = final_metadata  # type: ignore[attr-defined]
        cls._registry_name = reg_name  # type: ignore[attr-defined]
        cls._action_metadata = final_metadata  # type: ignore[attr-defined]  # Legacy compatibility

        logger.debug(f"Decorated and registered '{final_item_name}' in '{reg_name}'")
        return cls

    return decorator


def get_registry(name: str) -> Registry[Any]:
    """Get a registry by name.

    Args:
        name: Registry identifier

    Returns
    -------
        The registry instance

    Example:
        commands = get_registry("commands")
        build_class = commands.get("build")
    """
    return Registry(name)



"""Tests for the Registry module."""

import threading
from dataclasses import dataclass

import pytest
from pytest import LogCaptureFixture

from pytola_core.patterns.registry import DefaultRegistry, Registry, get_registry, register


class TestRegistry:
    """Test cases for the Registry class."""

    # Basic Registry Creation Tests
    def test_create_registry(self) -> None:
        """Test creating a registry."""
        registry = Registry("test")
        assert registry.name == "test"

    def test_registry_singleton(self) -> None:
        """Test that registries with the same name are singletons."""
        registry1 = Registry("singleton_test")
        registry2 = Registry("singleton_test")
        assert registry1 is registry2

    def test_multiple_registries(self) -> None:
        """Test creating multiple independent registries."""
        registry1 = Registry("test1")
        registry2 = Registry("test2")
        assert registry1 is not registry2
        assert registry1.name == "test1"
        assert registry2.name == "test2"

    # Registration Tests
    def test_register_class(self) -> None:
        """Test registering a class."""

        class TestClass:
            pass

        registry = Registry("register_test")
        registry.register("test_class", TestClass)

        retrieved = registry.get("test_class")
        assert retrieved is TestClass

    def test_register_with_metadata(self) -> None:
        """Test registering a class with metadata."""

        class TestClass:
            pass

        registry = Registry("metadata_test")
        metadata = {"description": "Test class", "version": "1.0"}
        registry.register("test_class", TestClass, metadata=metadata)

        retrieved_metadata = registry.get_metadata("test_class")
        assert retrieved_metadata["description"] == "Test class"
        assert retrieved_metadata["version"] == "1.0"

    def test_register_with_aliases(self) -> None:
        """Test registering a class with aliases."""

        class TestClass:
            pass

        registry = Registry("alias_test")
        registry.register("test_class", TestClass, aliases=["tc", "t"])

        # Get by canonical name
        assert registry.get("test_class") is TestClass
        # Get by alias
        assert registry.get("tc") is TestClass
        assert registry.get("t") is TestClass

    def test_register_overwrite_warning(self, caplog: LogCaptureFixture) -> None:
        """Test that overwriting a registration produces a warning."""

        class TestClass1:
            pass

        class TestClass2:
            pass

        registry = Registry("overwrite_test")
        registry.register("test_class", TestClass1)
        registry.register("test_class", TestClass2)

        assert "already registered" in caplog.text
        assert registry.get("test_class") is TestClass2

    def test_alias_overwrite_warning(self, caplog: LogCaptureFixture) -> None:
        """Test that overwriting an alias produces a warning."""

        class TestClass1:
            pass

        class TestClass2:
            pass

        registry = Registry("alias_overwrite_test")
        registry.register("class1", TestClass1, aliases=["a"])
        registry.register("class2", TestClass2, aliases=["a"])

        assert "already registered" in caplog.text
        assert registry.get("a") is TestClass2

    # Input Validation Tests
    def test_register_empty_name(self) -> None:
        """Test that empty registration name raises ValueError."""
        registry = Registry("validation_test")

        with pytest.raises(ValueError, match="cannot be empty"):
            registry.register("", object)

    def test_register_long_name(self) -> None:
        """Test that long registration name raises ValueError."""
        registry = Registry("validation_test")
        long_name = "x" * 100

        with pytest.raises(ValueError, match="exceeds maximum length"):
            registry.register(long_name, object)

    def test_register_empty_alias(self) -> None:
        """Test that empty alias raises ValueError."""
        registry = Registry("validation_test")

        with pytest.raises(ValueError, match="cannot be empty string"):
            registry.register("valid", object, aliases=[""])

    def test_register_long_alias(self) -> None:
        """Test that long alias raises ValueError."""
        registry = Registry("validation_test")
        long_alias = "y" * 100

        with pytest.raises(ValueError, match="exceeds maximum length"):
            registry.register("valid", object, aliases=[long_alias])

    # Unregister Tests
    def test_unregister_class(self) -> None:
        """Test unregistering a class."""

        class TestClass:
            pass

        registry = Registry("unregister_test")
        registry.register("test_class", TestClass)

        # Verify it's registered
        assert registry.get("test_class") is TestClass

        # Unregister it
        registry.unregister("test_class")

        # Verify it's gone
        assert registry.get("test_class") is None

    def test_unregister_removes_aliases(self) -> None:
        """Test that unregistering removes associated aliases."""

        class TestClass:
            pass

        registry = Registry("unregister_alias_test")
        registry.register("test_class", TestClass, aliases=["tc", "t"])

        # Verify aliases work
        assert registry.get("tc") is TestClass

        # Unregister
        registry.unregister("test_class")

        # Verify aliases are removed
        assert registry.get("tc") is None
        assert registry.get("t") is None

    def test_unregister_non_existent_warning(self, caplog: LogCaptureFixture) -> None:
        """Test that unregistering non-existent item produces a warning."""
        registry = Registry("nonexistent_test")
        registry.unregister("does_not_exist")

        assert "non-existent" in caplog.text

    def test_unregister_preserves_other_items(self) -> None:
        """Test that unregistering one item doesn't affect others."""

        class TestClass1:
            pass

        class TestClass2:
            pass

        registry = Registry("preserve_test")
        registry.register("class1", TestClass1)
        registry.register("class2", TestClass2)

        registry.unregister("class1")

        # class2 should still be registered
        assert registry.get("class2") is TestClass2
        assert registry.get("class1") is None

    # Get Methods Tests
    def test_get_non_existent(self) -> None:
        """Test getting a non-existent item returns None."""
        registry = Registry("get_test")
        result = registry.get("does_not_exist")
        assert result is None

    def test_get_instance(self) -> None:
        """Test getting an instance of a registered class."""

        @dataclass
        class TestClass:
            value: int = 42

        registry = Registry("instance_test")
        registry.register("test_class", TestClass)

        instance = registry.get_instance("test_class")
        assert isinstance(instance, TestClass)
        assert instance.value == 42

    def test_get_instance_with_args(self) -> None:
        """Test getting an instance with constructor arguments."""

        @dataclass
        class TestClass:
            value: int

        registry = Registry("instance_args_test")
        registry.register("test_class", TestClass)

        instance = registry.get_instance("test_class", 100)
        assert isinstance(instance, TestClass)
        assert instance.value == 100

    def test_get_instance_non_existent(self) -> None:
        """Test getting an instance of non-existent class returns None."""
        registry = Registry("instance_none_test")
        result = registry.get_instance("does_not_exist")
        assert result is None

    # List Methods Tests
    def test_get_classes(self) -> None:
        """Test getting all registered classes."""

        class Class1:
            pass

        class Class2:
            pass

        registry = Registry("list_test")
        registry.register("class1", Class1)
        registry.register("class2", Class2)

        classes = registry.get_classes()
        assert len(classes) == 2
        assert Class1 in classes
        assert Class2 in classes

    def test_list_items(self) -> None:
        """Test listing all registered item names."""

        class Class1:
            pass

        class Class2:
            pass

        registry = Registry("list_names_test")
        registry.register("class1", Class1)
        registry.register("class2", Class2)

        names = registry.list_items()
        assert len(names) == 2
        assert "class1" in names
        assert "class2" in names

    def test_list_all(self) -> None:
        """Test listing all items with metadata."""

        class Class1:
            pass

        registry = Registry("list_all_test")
        metadata = {"description": "Test"}
        registry.register("class1", Class1, aliases=["c1"], metadata=metadata)

        all_items = registry.list_all()
        assert len(all_items) == 1
        assert "class1" in all_items
        assert all_items["class1"]["description"] == "Test"
        assert all_items["class1"]["class"] is Class1
        assert "c1" in all_items["class1"]["aliases"]

    def test_get_metadata_default(self) -> None:
        """Test getting metadata returns empty dict for non-existent."""
        registry = Registry("metadata_default_test")
        metadata = registry.get_metadata("does_not_exist")
        assert metadata == {}

    # Special Methods Tests
    def test_len(self) -> None:
        """Test __len__ method."""
        registry = Registry("len_test")
        assert len(registry) == 0

        registry.register("class1", object)
        assert len(registry) == 1

        registry.register("class2", object)
        assert len(registry) == 2

    def test_contains_by_name(self) -> None:
        """Test __contains__ method with canonical name."""
        registry = Registry("contains_test")
        registry.register("test_class", object)

        assert "test_class" in registry
        assert "other_class" not in registry

    def test_contains_by_alias(self) -> None:
        """Test __contains__ method with alias."""
        registry = Registry("contains_alias_test")
        registry.register("test_class", object, aliases=["tc"])

        assert "tc" in registry
        assert "test_class" in registry

    # Clear Tests
    def test_clear(self) -> None:
        """Test clearing a registry."""

        class TestClass:
            pass

        registry = Registry("clear_test")
        registry.register("test_class", TestClass, aliases=["tc"])

        registry.clear()

        assert len(registry) == 0
        assert registry.get("test_class") is None
        assert registry.get("tc") is None

    def test_clear_all(self) -> None:
        """Test clearing all registries."""
        registry1 = Registry("clear_all_1")
        registry2 = Registry("clear_all_2")

        registry1.register("class1", object)
        registry2.register("class2", object)

        Registry.clear_all()

        assert len(registry1) == 0
        assert len(registry2) == 0

    # Thread Safety Tests
    def test_thread_safe_creation(self) -> None:
        """Test thread-safe registry creation."""
        registries = []
        lock = threading.Lock()

        def create_registry(name: str) -> None:
            reg = Registry(f"thread_{name}")
            with lock:
                registries.append(reg)

        threads = [threading.Thread(target=create_registry, args=(str(i),)) for i in range(5)]

        for thread in threads:
            thread.start()
        for thread in threads:
            thread.join()

        assert len(registries) == 5
        # All should be unique
        names = [reg.name for reg in registries]
        assert len(set(names)) == 5

    def test_thread_safe_initialization(self) -> None:
        """Test thread-safe registry initialization."""
        Registry("thread_init_test")

        def access_registry() -> None:
            # Multiple threads accessing the same registry
            _ = Registry("thread_init_test")

        threads = [threading.Thread(target=access_registry) for _ in range(5)]

        for thread in threads:
            thread.start()
        for thread in threads:
            thread.join()

        # Should not raise any exceptions

    # Class Method Tests
    def test_get_registry_class_method(self) -> None:
        """Test get_registry class method."""
        registry = Registry.get_registry("class_method_test")
        assert registry.name == "class_method_test"

    def test_list_registries(self) -> None:
        """Test list_registries class method."""
        Registry.clear_all()

        Registry("registry1")
        Registry("registry2")

        registry_names = Registry.list_registries()
        assert "registry1" in registry_names
        assert "registry2" in registry_names


class TestDefaultRegistry:
    """Test cases for the DefaultRegistry class."""

    def test_default_registry_singleton(self) -> None:
        """Test that DefaultRegistry is a singleton."""
        default1 = DefaultRegistry()
        default2 = DefaultRegistry()
        assert default1 is default2

    def test_default_registry_get_default_registry(self) -> None:
        """Test get_default_registry class method."""
        default1 = DefaultRegistry.get_default_registry()
        default2 = DefaultRegistry.get_default_registry()
        assert default1 is default2

    def test_default_registry_name(self) -> None:
        """Test that default registry has name 'default'."""
        default = DefaultRegistry()
        assert default.name == "default"

    def test_default_registry_backward_compatibility(self) -> None:
        """Test backward compatibility with regular Registry."""
        default = DefaultRegistry()
        also_default = Registry("default")
        assert default is also_default


class TestRegisterDecorator:
    """Test cases for the register decorator."""

    def test_decorator_new_style(self) -> None:
        """Test new style decorator with registry name."""

        @register("decorator_test", name="my_class")
        class MyClass:
            pass

        registry = Registry("decorator_test")
        assert registry.get("my_class") is MyClass

    def test_decorator_default_name(self) -> None:
        """Test decorator using class name as default."""

        @register("default_name_test")
        class AutoNamedClass:
            pass

        registry = Registry("default_name_test")
        assert registry.get("autonamedclass") is AutoNamedClass

    def test_decorator_with_aliases(self) -> None:
        """Test decorator with aliases."""

        @register("alias_dec_test", name="my_class", aliases=["mc", "m"])
        class MyClass:
            pass

        registry = Registry("alias_dec_test")
        assert registry.get("my_class") is MyClass
        assert registry.get("mc") is MyClass
        assert registry.get("m") is MyClass

    def test_decorator_with_metadata(self) -> None:
        """Test decorator with metadata."""

        @register("metadata_dec_test", name="my_class", description="A test class")
        class MyClass:
            pass

        registry = Registry("metadata_dec_test")
        metadata = registry.get_metadata("my_class")
        assert metadata["description"] == "A test class"
        assert metadata["class_name"] == "MyClass"

    def test_decorator_legacy_style(self) -> None:
        """Test legacy style decorator without registry name."""

        @register(name="legacy_class", aliases=["lc"])
        class LegacyClass:
            pass

        registry = Registry("default")
        assert registry.get("legacy_class") is LegacyClass
        assert registry.get("lc") is LegacyClass

    def test_decorator_class_metadata_attribute(self) -> None:
        """Test that decorator stores metadata on class."""

        @register("attr_test", name="my_class", description="Test")
        class MyClass:
            pass

        assert hasattr(MyClass, "_registry_metadata")
        assert MyClass._registry_metadata["description"] == "Test"
        assert MyClass._registry_name == "attr_test"


class TestGetRegistry:
    """Test cases for the get_registry function."""

    def test_get_registry_function(self) -> None:
        """Test get_registry convenience function."""
        registry = get_registry("function_test")
        assert registry.name == "function_test"

        # Should be the same as calling Registry directly
        same_registry = Registry("function_test")
        assert registry is same_registry


class TestMetadataFormat:
    """Test cases for metadata format."""

    def test_metadata_structure(self) -> None:
        """Test registry metadata structure."""
        metadata = {
            "name": "test",
            "class_name": "TestClass",
            "description": "A test class",
            "help_text": "Help text here",
            "aliases": ["t", "tc"],
        }

        assert metadata["name"] == "test"
        assert metadata["class_name"] == "TestClass"
        assert metadata["description"] == "A test class"
        assert metadata["help_text"] == "Help text here"
        assert metadata["aliases"] == ["t", "tc"]

    def test_metadata_optional_fields(self) -> None:
        """Test that metadata fields are optional."""
        # Should be able to create partial metadata
        metadata = {"name": "test", "class_name": "TestClass"}

        assert metadata["name"] == "test"
        assert metadata["class_name"] == "TestClass"


class TestEdgeCases:
    """Test edge cases and boundary conditions."""

    def test_max_length_name(self) -> None:
        """Test registration with maximum length name."""
        registry = Registry("max_length_test")
        max_length_name = "x" * 64  # Exactly at the limit

        # Should not raise
        registry.register(max_length_name, object)
        assert registry.get(max_length_name) is object

    def test_max_length_alias(self) -> None:
        """Test registration with maximum length alias."""
        registry = Registry("max_alias_test")
        max_length_alias = "y" * 64  # Exactly at the limit

        # Should not raise
        registry.register("valid", object, aliases=[max_length_alias])
        assert registry.get(max_length_alias) is object

    def test_special_characters_in_name(self) -> None:
        """Test registration with special characters in name."""
        registry = Registry("special_chars_test")
        special_name = "test-class_123.test/class"

        # Should accept special characters
        registry.register(special_name, object)
        assert registry.get(special_name) is object

    def test_unicode_in_name(self) -> None:
        """Test registration with unicode characters."""
        registry = Registry("unicode_test")
        unicode_name = "测试类_テスト"

        registry.register(unicode_name, object)
        assert registry.get(unicode_name) is object

    def test_case_sensitivity(self) -> None:
        """Test that names are case-sensitive."""
        registry = Registry("case_test")

        class Class1:
            pass

        class Class2:
            pass

        registry.register("TestClass", Class1)
        registry.register("testclass", Class2)

        # Should be different registrations
        assert registry.get("TestClass") is Class1
        assert registry.get("testclass") is Class2
        assert len(registry) == 2

    def test_get_instance_with_kwargs(self) -> None:
        """Test getting instance with keyword arguments."""

        class TestClass:
            def __init__(self, value: int = 10, multiplier: int = 1) -> None:
                self.result = value * multiplier

        registry = Registry("kwargs_test")
        registry.register("test", TestClass)

        instance = registry.get_instance("test", value=5, multiplier=3)
        assert instance.result == 15



"""Pytest configuration for pytola-core tests."""

import logging
from typing import Generator

import pytest

from pytola_core.patterns.registry import Registry


@pytest.fixture(autouse=True)
def _clear_registries() -> Generator[None, None, None]:
    """Clear all registries before and after each test automatically.

    This fixture is auto-used by all tests to ensure clean state between tests.
    It runs once per test function, clearing registries both before and after.
    """
    Registry.clear_all()
    yield
    Registry.clear_all()


@pytest.fixture(autouse=True)
def _enable_log_propagation(caplog: pytest.LogCaptureFixture) -> Generator[None, None, None]:
    """Enable log propagation for caplog to work properly.

    The logger module sets propagate=False which prevents caplog from
    capturing logs. This fixture temporarily enables propagation during tests.
    """
    # Get the logger instance used in the code
    log = logging.getLogger("pytola")
    original_propagate = log.propagate

    # Enable propagation so caplog can capture logs
    log.propagate = True

    yield

    # Restore original setting
    log.propagate = original_propagate
